@@ -5,18 +5,18 @@
  */
 
 mergeInto(LibraryManager.library, {
+  emscripten_get_exported_function__sig: 'pp',
+  emscripten_get_exported_function__deps: ['$addFunction'],
   emscripten_get_exported_function: function(name) {
     name = UTF8ToString(name);
-#if WASM_BACKEND
     // Wasm backend does not use C name mangling on exports,
     // so adjust for that manually.
     if (name[0] == '_') name = name.substr(1);
-#endif
 #if MINIMAL_RUNTIME
     var exportedFunc = asm[name];
 #else
     // In regular runtime, exports are available on the Module object.
-    var exportedFunc = Module["asm"][name];
+    var exportedFunc = wasmExports[name];
 #endif
     if (exportedFunc) {
       // Record the created function pointer to each function object,
@@ -26,7 +26,7 @@ mergeInto(LibraryManager.library, {
       return exportedFunc.ptr;
     }
 #if ASSERTIONS
-    console.error('No exported function found by name "' + exportedFunc + '"');
+    err('No exported function found by name "' + exportedFunc + '"');
 #endif
     // implicit return 0;
   }
