@@ -6,21 +6,21 @@
 
 #if LZ4
 mergeInto(LibraryManager.library, {
-  $LZ4__deps: ['$FS', '$ERRNO_CODES'],
+  $LZ4__deps: ['$FS', '$preloadPlugins'],
   $LZ4: {
-    DIR_MODE: {{{ cDefine('S_IFDIR') }}} | 511 /* 0777 */,
-    FILE_MODE: {{{ cDefine('S_IFREG') }}} | 511 /* 0777 */,
+    DIR_MODE: {{{ cDefs.S_IFDIR }}} | 511 /* 0777 */,
+    FILE_MODE: {{{ cDefs.S_IFREG }}} | 511 /* 0777 */,
     CHUNK_SIZE: -1,
     codec: null,
-    init: function() {
+    init() {
       if (LZ4.codec) return;
       LZ4.codec = (function() {
-        {{{ read('mini-lz4.js') }}};
+        {{{ read('../third_party/mini-lz4.js') }}};
         return MiniLZ4;
       })();
       LZ4.CHUNK_SIZE = LZ4.codec.CHUNK_SIZE;
     },
-    loadPackage: function (pack) {
+    loadPackage(pack, preloadPlugin) {
       LZ4.init();
       var compressedData = pack['compressedData'];
       if (!compressedData) compressedData = LZ4.codec.compressPackage(pack['data']);
@@ -37,13 +37,36 @@ mergeInto(LibraryManager.library, {
         FS.createPath('', dir, true, true);
         var parent = FS.analyzePath(dir).object;
         LZ4.createNode(parent, name, LZ4.FILE_MODE, 0, {
-          compressedData: compressedData,
+          compressedData,
           start: file.start,
           end: file.end,
         });
       });
+      // Preload files if necessary. This code is largely similar to
+      // createPreloadedFile in library_fs.js. However, a main difference here
+      // is that we only decompress the file if it can be preloaded.
+      // Abstracting out the common parts seems to be more effort than it is
+      // worth.
+      if (preloadPlugin) {
+        Browser.init();
+        pack['metadata'].files.forEach(function(file) {
+          var handled = false;
+          var fullname = file.filename;
+          preloadPlugins.forEach(function(plugin) {
+            if (handled) return;
+            if (plugin['canHandle'](fullname)) {
+              var dep = getUniqueRunDependency('fp ' + fullname);
+              addRunDependency(dep);
+              var finish = () => removeRunDependency(dep);
+              var byteArray = FS.readFile(fullname);
+              plugin['handle'](byteArray, fullname, finish, finish);
+              handled = true;
+            }
+          });
+        });
+      }
     },
-    createNode: function (parent, name, mode, dev, contents, mtime) {
+    createNode(parent, name, mode, dev, contents, mtime) {
       var node = FS.createNode(parent, name, mode);
       node.mode = mode;
       node.node_ops = LZ4.node_ops;
@@ -63,7 +86,7 @@ mergeInto(LibraryManager.library, {
       return node;
     },
     node_ops: {
-      getattr: function(node) {
+      getattr(node) {
         return {
           dev: 1,
           ino: node.id,
@@ -71,7 +94,7 @@ mergeInto(LibraryManager.library, {
           nlink: 1,
           uid: 0,
           gid: 0,
-          rdev: undefined,
+          rdev: 0,
           size: node.size,
           atime: new Date(node.timestamp),
           mtime: new Date(node.timestamp),
@@ -80,7 +103,7 @@ mergeInto(LibraryManager.library, {
           blocks: Math.ceil(node.size / 4096),
         };
       },
-      setattr: function(node, attr) {
+      setattr(node, attr) {
         if (attr.mode !== undefined) {
           node.mode = attr.mode;
         }
@@ -88,34 +111,31 @@ mergeInto(LibraryManager.library, {
           node.timestamp = attr.timestamp;
         }
       },
-      lookup: function(parent, name) {
-        throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
+      lookup(parent, name) {
+        throw new FS.ErrnoError({{{ cDefs.ENOENT }}});
       },
-      mknod: function (parent, name, mode, dev) {
-        throw new FS.ErrnoError(ERRNO_CODES.EPERM);
+      mknod(parent, name, mode, dev) {
+        throw new FS.ErrnoError({{{ cDefs.EPERM }}});
       },
-      rename: function (oldNode, newDir, newName) {
-        throw new FS.ErrnoError(ERRNO_CODES.EPERM);
+      rename(oldNode, newDir, newName) {
+        throw new FS.ErrnoError({{{ cDefs.EPERM }}});
       },
-      unlink: function(parent, name) {
-        throw new FS.ErrnoError(ERRNO_CODES.EPERM);
+      unlink(parent, name) {
+        throw new FS.ErrnoError({{{ cDefs.EPERM }}});
       },
-      rmdir: function(parent, name) {
-        throw new FS.ErrnoError(ERRNO_CODES.EPERM);
+      rmdir(parent, name) {
+        throw new FS.ErrnoError({{{ cDefs.EPERM }}});
       },
-      readdir: function(node) {
-        throw new FS.ErrnoError(ERRNO_CODES.EPERM);
+      readdir(node) {
+        throw new FS.ErrnoError({{{ cDefs.EPERM }}});
       },
-      symlink: function(parent, newName, oldPath) {
-        throw new FS.ErrnoError(ERRNO_CODES.EPERM);
-      },
-      readlink: function(node) {
-        throw new FS.ErrnoError(ERRNO_CODES.EPERM);
+      symlink(parent, newName, oldPath) {
+        throw new FS.ErrnoError({{{ cDefs.EPERM }}});
       },
     },
     stream_ops: {
-      read: function (stream, buffer, offset, length, position) {
-        //console.log('LZ4 read ' + [offset, length, position]);
+      read(stream, buffer, offset, length, position) {
+        //out('LZ4 read ' + [offset, length, position]);
         length = Math.min(length, stream.node.size - position);
         if (length <= 0) return 0;
         var contents = stream.node.contents;
@@ -124,7 +144,7 @@ mergeInto(LibraryManager.library, {
         while (written < length) {
           var start = contents.start + position + written; // start index in uncompressed data
           var desired = length - written;
-          //console.log('current read: ' + ['start', start, 'desired', desired]);
+          //out('current read: ' + ['start', start, 'desired', desired]);
           var chunkIndex = Math.floor(start / LZ4.CHUNK_SIZE);
           var compressedStart = compressedData['offsets'][chunkIndex];
           var compressedSize = compressedData['sizes'][chunkIndex];
@@ -140,13 +160,13 @@ mergeInto(LibraryManager.library, {
               currChunk = compressedData['cachedChunks'].pop();
               compressedData['cachedChunks'].unshift(currChunk);
               if (compressedData['debug']) {
-                console.log('decompressing chunk ' + chunkIndex);
+                out('decompressing chunk ' + chunkIndex);
                 Module['decompressedChunks'] = (Module['decompressedChunks'] || 0) + 1;
               }
               var compressed = compressedData['data'].subarray(compressedStart, compressedStart + compressedSize);
               //var t = Date.now();
               var originalSize = LZ4.codec.uncompress(compressed, currChunk);
-              //console.log('decompress time: ' + (Date.now() - t));
+              //out('decompress time: ' + (Date.now() - t));
               if (chunkIndex < compressedData['successes'].length-1) assert(originalSize === LZ4.CHUNK_SIZE); // all but the last chunk must be full-size
             }
           } else {
@@ -161,20 +181,20 @@ mergeInto(LibraryManager.library, {
         }
         return written;
       },
-      write: function (stream, buffer, offset, length, position) {
-        throw new FS.ErrnoError(ERRNO_CODES.EIO);
+      write(stream, buffer, offset, length, position) {
+        throw new FS.ErrnoError({{{ cDefs.EIO }}});
       },
-      llseek: function (stream, offset, whence) {
+      llseek(stream, offset, whence) {
         var position = offset;
-        if (whence === {{{ cDefine('SEEK_CUR') }}}) {
+        if (whence === {{{ cDefs.SEEK_CUR }}}) {
           position += stream.position;
-        } else if (whence === {{{ cDefine('SEEK_END') }}}) {
+        } else if (whence === {{{ cDefs.SEEK_END }}}) {
           if (FS.isFile(stream.node.mode)) {
             position += stream.node.size;
           }
         }
         if (position < 0) {
-          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
+          throw new FS.ErrnoError({{{ cDefs.EINVAL }}});
         }
         return position;
       },
