@@ -5,25 +5,23 @@
  */
 
 var LibraryStackTrace = {
-  $demangle__deps: ['$withStackSave'],
+#if DEMANGLE_SUPPORT
+  $demangle__deps: ['$withStackSave', '__cxa_demangle', 'free', '$stringToUTF8OnStack'],
+#endif
   $demangle: function(func) {
 #if DEMANGLE_SUPPORT
     // If demangle has failed before, stop demangling any further function names
     // This avoids an infinite recursion with malloc()->abort()->stackTrace()->demangle()->malloc()->...
     demangle.recursionGuard = (demangle.recursionGuard|0)+1;
     if (demangle.recursionGuard > 1) return func;
-    var __cxa_demangle_func = Module['___cxa_demangle'] || Module['__cxa_demangle'];
-    assert(__cxa_demangle_func);
     return withStackSave(function() {
       try {
         var s = func;
         if (s.startsWith('__Z'))
           s = s.substr(1);
-        var len = lengthBytesUTF8(s)+1;
-        var buf = stackAlloc(len);
-        stringToUTF8(s, buf, len);
+        var buf = stringToUTF8OnStack(s);
         var status = stackAlloc(4);
-        var ret = __cxa_demangle_func(buf, 0, 0, status);
+        var ret = ___cxa_demangle(buf, 0, 0, status);
         if ({{{ makeGetValue('status', '0', 'i32') }}} === 0 && ret) {
           return UTF8ToString(ret);
         }
@@ -44,6 +42,7 @@ var LibraryStackTrace = {
 #endif // DEMANGLE_SUPPORT
   },
 
+  $demangleAll__deps: ['$demangle'],
   $demangleAll: function(text) {
     var regex =
       /\b_Z[\w\d_]+/g;
@@ -71,6 +70,7 @@ var LibraryStackTrace = {
     return error.stack.toString();
   },
 
+  $stackTrace__deps: ['$jsStackTrace', '$demangleAll'],
   $stackTrace: function() {
     var js = jsStackTrace();
     if (Module['extraStackTrace']) js += '\n' + Module['extraStackTrace']();
