@@ -6,9 +6,9 @@
 
 var LibraryTracing = {
   $EmscriptenTrace__deps: [
-    'emscripten_trace_js_configure', 'emscripten_trace_configure_for_google_wtf',
-    'emscripten_trace_js_enter_context', 'emscripten_trace_exit_context',
-    'emscripten_trace_js_log_message', 'emscripten_trace_js_mark',
+    '$traceConfigure', 'emscripten_trace_configure_for_google_wtf',
+    '$traceEnterContext', 'emscripten_trace_exit_context',
+    '$traceLogMessage', '$traceMark',
     'emscripten_get_now'
   ],
   $EmscriptenTrace__postset: 'EmscriptenTrace.init()',
@@ -49,12 +49,12 @@ var LibraryTracing = {
     EVENT_USER_NAME: 'user-name',
 
     init: function() {
-      Module['emscripten_trace_configure'] = _emscripten_trace_js_configure;
+      Module['emscripten_trace_configure'] = traceConfigure;
       Module['emscripten_trace_configure_for_google_wtf'] = _emscripten_trace_configure_for_google_wtf;
-      Module['emscripten_trace_enter_context'] = _emscripten_trace_js_enter_context;
+      Module['emscripten_trace_enter_context'] = traceEnterContext;
       Module['emscripten_trace_exit_context'] = _emscripten_trace_exit_context;
-      Module['emscripten_trace_log_message'] = _emscripten_trace_js_log_message;
-      Module['emscripten_trace_mark'] = _emscripten_trace_js_mark;
+      Module['emscripten_trace_log_message'] = traceLogMessage;
+      Module['emscripten_trace_mark'] = traceMark;
     },
 
     // Work around CORS issues ...
@@ -79,8 +79,8 @@ var LibraryTracing = {
       EmscriptenTrace.loadWorkerViaXHR(collector_url + 'worker.js', function (worker) {
         EmscriptenTrace.worker = worker;
         EmscriptenTrace.worker.addEventListener('error', function (e) {
-          console.log('TRACE WORKER ERROR:');
-          console.log(e);
+          out('TRACE WORKER ERROR:');
+          out(e);
         }, false);
         EmscriptenTrace.worker.postMessage({ 'cmd': 'configure',
                                              'data_version': EmscriptenTrace.DATA_VERSION,
@@ -104,7 +104,7 @@ var LibraryTracing = {
       if (window && window['wtf']) {
         EmscriptenTrace.googleWTFEnabled = true;
       } else {
-        console.log('GOOGLE WTF NOT AVAILABLE TO ENABLE');
+        out('GOOGLE WTF NOT AVAILABLE TO ENABLE');
       }
     },
 
@@ -133,7 +133,7 @@ var LibraryTracing = {
     }
   },
 
-  emscripten_trace_js_configure: function(collector_url, application) {
+  $traceConfigure: function(collector_url, application) {
     EmscriptenTrace.configure(collector_url, application);
   },
 
@@ -172,7 +172,7 @@ var LibraryTracing = {
     }
   },
 
-  emscripten_trace_js_log_message: function(channel, message) {
+  $traceLogMessage: function(channel, message) {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_LOG_MESSAGE, now,
@@ -189,7 +189,7 @@ var LibraryTracing = {
     }
   },
 
-  emscripten_trace_js_mark: function(message) {
+  $traceMark: function(message) {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_LOG_MESSAGE, now,
@@ -219,7 +219,7 @@ var LibraryTracing = {
   },
 
   emscripten_trace_record_allocation: function(address, size) {
-    if (typeof Module['onMalloc'] === 'function') Module['onMalloc'](address, size);
+    if (typeof Module['onMalloc'] == 'function') Module['onMalloc'](address, size);
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_ALLOCATE,
@@ -228,7 +228,7 @@ var LibraryTracing = {
   },
 
   emscripten_trace_record_reallocation: function(old_address, new_address, size) {
-    if (typeof Module['onRealloc'] === 'function') Module['onRealloc'](old_address, new_address, size);
+    if (typeof Module['onRealloc'] == 'function') Module['onRealloc'](old_address, new_address, size);
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_REALLOCATE,
@@ -237,7 +237,7 @@ var LibraryTracing = {
   },
 
   emscripten_trace_record_free: function(address) {
-    if (typeof Module['onFree'] === 'function') Module['onFree'](address);
+    if (typeof Module['onFree'] == 'function') Module['onFree'](address);
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_FREE,
@@ -263,10 +263,10 @@ var LibraryTracing = {
     if (EmscriptenTrace.postEnabled) {
       var memory_layout = {
         'static_base':  {{{ GLOBAL_BASE }}},
-        'stack_base':   STACK_BASE,
-        'stack_top':    STACKTOP,
-        'stack_max':    STACK_MAX,
-        'dynamic_top':  _sbrk(),
+        'stack_base':   _emscripten_stack_get_base(),
+        'stack_top':    _emscripten_stack_get_current(),
+        'stack_max':    _emscripten_stack_get_end(),
+        'dynamic_top':  _sbrk(0),
         'total_memory': HEAP8.length
       };
       var now = EmscriptenTrace.now();
@@ -298,7 +298,7 @@ var LibraryTracing = {
     }
   },
 
-  emscripten_trace_js_enter_context: function(name) {
+  $traceEnterContext: function(name) {
     if (EmscriptenTrace.postEnabled) {
       var now = EmscriptenTrace.now();
       EmscriptenTrace.post([EmscriptenTrace.EVENT_ENTER_CONTEXT,
