@@ -1,10 +1,15 @@
-// Copyright 2013 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2013 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 mergeInto(LibraryManager.library, {
-  $TTY__deps: ['$FS'],
+  $TTY__deps: [
+    '$FS',
+    '$UTF8ArrayToString',
+    '$FS_stdin_getChar'
+  ],
 #if !MINIMAL_RUNTIME
   $TTY__postset: function() {
     addAtInit('TTY.init();');
@@ -20,7 +25,7 @@ mergeInto(LibraryManager.library, {
       //   // device, it always assumes it's a TTY device. because of this, we're forcing
       //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
       //   // with text files until FS.init can be refactored.
-      //   process['stdin']['setEncoding']('utf8');
+      //   process.stdin.setEncoding('utf8');
       // }
     },
     shutdown: function() {
@@ -31,7 +36,7 @@ mergeInto(LibraryManager.library, {
       //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
       //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
       //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
-      //   process['stdin']['pause']();
+      //   process.stdin.pause();
       // }
     },
     register: function(dev, ops) {
@@ -42,21 +47,21 @@ mergeInto(LibraryManager.library, {
       open: function(stream) {
         var tty = TTY.ttys[stream.node.rdev];
         if (!tty) {
-          throw new FS.ErrnoError({{{ cDefine('ENODEV') }}});
+          throw new FS.ErrnoError({{{ cDefs.ENODEV }}});
         }
         stream.tty = tty;
         stream.seekable = false;
       },
       close: function(stream) {
         // flush any pending line data
-        stream.tty.ops.flush(stream.tty);
+        stream.tty.ops.fsync(stream.tty);
       },
-      flush: function(stream) {
-        stream.tty.ops.flush(stream.tty);
+      fsync: function(stream) {
+        stream.tty.ops.fsync(stream.tty);
       },
       read: function(stream, buffer, offset, length, pos /* ignored */) {
         if (!stream.tty || !stream.tty.ops.get_char) {
-          throw new FS.ErrnoError({{{ cDefine('ENXIO') }}});
+          throw new FS.ErrnoError({{{ cDefs.ENXIO }}});
         }
         var bytesRead = 0;
         for (var i = 0; i < length; i++) {
@@ -64,10 +69,10 @@ mergeInto(LibraryManager.library, {
           try {
             result = stream.tty.ops.get_char(stream.tty);
           } catch (e) {
-            throw new FS.ErrnoError({{{ cDefine('EIO') }}});
+            throw new FS.ErrnoError({{{ cDefs.EIO }}});
           }
           if (result === undefined && bytesRead === 0) {
-            throw new FS.ErrnoError({{{ cDefine('EAGAIN') }}});
+            throw new FS.ErrnoError({{{ cDefs.EAGAIN }}});
           }
           if (result === null || result === undefined) break;
           bytesRead++;
@@ -80,14 +85,14 @@ mergeInto(LibraryManager.library, {
       },
       write: function(stream, buffer, offset, length, pos) {
         if (!stream.tty || !stream.tty.ops.put_char) {
-          throw new FS.ErrnoError({{{ cDefine('ENXIO') }}});
+          throw new FS.ErrnoError({{{ cDefs.ENXIO }}});
         }
         try {
           for (var i = 0; i < length; i++) {
             stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
           }
         } catch (e) {
-          throw new FS.ErrnoError({{{ cDefine('EIO') }}});
+          throw new FS.ErrnoError({{{ cDefs.EIO }}});
         }
         if (length) {
           stream.node.timestamp = Date.now();
@@ -96,69 +101,8 @@ mergeInto(LibraryManager.library, {
       }
     },
     default_tty_ops: {
-      // get_char has 3 particular return values:
-      // a.) the next character represented as an integer
-      // b.) undefined to signal that no data is currently available
-      // c.) null to signal an EOF
       get_char: function(tty) {
-        if (!tty.input.length) {
-          var result = null;
-#if ENVIRONMENT_MAY_BE_NODE
-          if (ENVIRONMENT_IS_NODE) {
-            // we will read data by chunks of BUFSIZE
-            var BUFSIZE = 256;
-            var buf = new Buffer(BUFSIZE);
-            var bytesRead = 0;
-
-            var isPosixPlatform = (process.platform != 'win32'); // Node doesn't offer a direct check, so test by exclusion
-
-            var fd = process.stdin.fd;
-            if (isPosixPlatform) {
-              // Linux and Mac cannot use process.stdin.fd (which isn't set up as sync)
-              var usingDevice = false;
-              try {
-                fd = fs.openSync('/dev/stdin', 'r');
-                usingDevice = true;
-              } catch (e) {}
-            }
-
-            try {
-              bytesRead = fs.readSync(fd, buf, 0, BUFSIZE, null);
-            } catch(e) {
-              // Cross-platform differences: on Windows, reading EOF throws an exception, but on other OSes,
-              // reading EOF returns 0. Uniformize behavior by treating the EOF exception to return 0.
-              if (e.toString().indexOf('EOF') != -1) bytesRead = 0;
-              else throw e;
-            }
-
-            if (usingDevice) { fs.closeSync(fd); }
-            if (bytesRead > 0) {
-              result = buf.slice(0, bytesRead).toString('utf-8');
-            } else {
-              result = null;
-            }
-          } else
-#endif
-          if (typeof window != 'undefined' &&
-            typeof window.prompt == 'function') {
-            // Browser.
-            result = window.prompt('Input: ');  // returns null on cancel
-            if (result !== null) {
-              result += '\n';
-            }
-          } else if (typeof readline == 'function') {
-            // Command line.
-            result = readline();
-            if (result !== null) {
-              result += '\n';
-            }
-          }
-          if (!result) {
-            return null;
-          }
-          tty.input = intArrayFromString(result, true);
-        }
-        return tty.input.shift();
+        return FS_stdin_getChar();
       },
       put_char: function(tty, val) {
         if (val === null || val === {{{ charCode('\n') }}}) {
@@ -168,11 +112,32 @@ mergeInto(LibraryManager.library, {
           if (val != 0) tty.output.push(val); // val == 0 would cut text output off in the middle.
         }
       },
-      flush: function(tty) {
+      fsync: function(tty) {
         if (tty.output && tty.output.length > 0) {
           out(UTF8ArrayToString(tty.output, 0));
           tty.output = [];
         }
+      },
+      ioctl_tcgets: function(tty) {
+        // typical setting
+        return {
+          c_iflag: {{{ cDefs.ICRNL | cDefs.IXON | cDefs.IMAXBEL | cDefs.IUTF8 }}},
+          c_oflag: {{{ cDefs.OPOST | cDefs.ONLCR }}},
+          c_cflag: {{{ cDefs.B38400 | cDefs.CSIZE | cDefs.CREAD }}},
+          c_lflag: {{{ cDefs.ISIG | cDefs.ICANON | cDefs.ECHO | cDefs.ECHOE | cDefs.ECHOK | cDefs.ECHOCTL | cDefs.ECHOKE | cDefs.IEXTEN }}},
+          c_cc: [
+            0x03, 0x1c, 0x7f, 0x15, 0x04, 0x00, 0x01, 0x00, 0x11, 0x13, 0x1a, 0x00,
+            0x12, 0x0f, 0x17, 0x16, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+          ]
+        };
+      },
+      ioctl_tcsets: function(tty, optional_actions, data) {
+        // currently just ignore
+        return 0;
+      },
+      ioctl_tiocgwinsz: function(tty) {
+        return [24, 80];
       }
     },
     default_tty1_ops: {
@@ -184,7 +149,7 @@ mergeInto(LibraryManager.library, {
           if (val != 0) tty.output.push(val);
         }
       },
-      flush: function(tty) {
+      fsync: function(tty) {
         if (tty.output && tty.output.length > 0) {
           err(UTF8ArrayToString(tty.output, 0));
           tty.output = [];
