@@ -26,18 +26,19 @@ global.LibraryManager = {
   loaded: false,
   libraries: [],
 
-  has: function(name) {
+  has(name) {
     return this.libraries.includes(name);
   },
 
-  load: function() {
+  load() {
     assert(!this.loaded);
     this.loaded = true;
 
     // Core system libraries (always linked against)
     let libraries = [
-      'library.js',
       'library_int53.js',
+      'library.js',
+      'library_sigs.js',
       'library_ccall.js',
       'library_addfunction.js',
       'library_formatString.js',
@@ -45,13 +46,12 @@ global.LibraryManager = {
       'library_math.js',
       'library_path.js',
       'library_strings.js',
-      'library_syscall.js',
       'library_html5.js',
       'library_stack_trace.js',
       'library_wasi.js',
-      'library_dylink.js',
       'library_makeDynCall.js',
       'library_eventloop.js',
+      'library_promise.js',
     ];
 
     if (LINK_AS_CXX) {
@@ -75,32 +75,45 @@ global.LibraryManager = {
       libraries.push('library_autodebug.js');
     }
 
-    if (FILESYSTEM) {
-      // Core filesystem libraries (always linked against, unless -sFILESYSTEM=0 is specified)
-      libraries = libraries.concat([
-        'library_fs.js',
-        'library_memfs.js',
-        'library_tty.js',
-        'library_pipefs.js', // ok to include it by default since it's only used if the syscall is used
-        'library_sockfs.js', // ok to include it by default since it's only used if the syscall is used
-      ]);
+    if (!WASMFS) {
+      libraries.push('library_syscall.js');
+    }
 
-      if (NODERAWFS) {
-        // NODERAWFS requires NODEFS
-        if (!JS_LIBRARIES.includes('library_nodefs.js')) {
-          libraries.push('library_nodefs.js');
+    if (RELOCATABLE) {
+      libraries.push('library_dylink.js');
+    }
+
+    if (FILESYSTEM) {
+      libraries.push('library_fs_shared.js');
+      if (WASMFS) {
+        libraries = libraries.concat([
+          'library_wasmfs.js',
+          'library_wasmfs_js_file.js',
+          'library_wasmfs_jsimpl.js',
+          'library_wasmfs_fetch.js',
+          'library_wasmfs_node.js',
+          'library_wasmfs_opfs.js',
+        ]);
+      } else {
+        // Core filesystem libraries (always linked against, unless -sFILESYSTEM=0 is specified)
+        libraries = libraries.concat([
+          'library_fs.js',
+          'library_memfs.js',
+          'library_tty.js',
+          'library_pipefs.js', // ok to include it by default since it's only used if the syscall is used
+          'library_sockfs.js', // ok to include it by default since it's only used if the syscall is used
+        ]);
+
+        if (NODERAWFS) {
+          // NODERAWFS requires NODEFS
+          if (!JS_LIBRARIES.includes('library_nodefs.js')) {
+            libraries.push('library_nodefs.js');
+          }
+          libraries.push('library_noderawfs.js');
+          // NODERAWFS overwrites library_path.js
+          libraries.push('library_nodepath.js');
         }
-        libraries.push('library_noderawfs.js');
-        // NODERAWFS overwrites library_path.js
-        libraries.push('library_nodepath.js');
       }
-    } else if (WASMFS) {
-      libraries.push('library_wasmfs.js');
-      libraries.push('library_wasmfs_js_file.js');
-      libraries.push('library_wasmfs_jsimpl.js');
-      libraries.push('library_wasmfs_fetch.js');
-      libraries.push('library_wasmfs_node.js');
-      libraries.push('library_wasmfs_opfs.js');
     }
 
     // Additional JS libraries (without AUTO_JS_LIBRARIES, link to these explicitly via -lxxx.js)
@@ -109,16 +122,17 @@ global.LibraryManager = {
         'library_webgl.js',
         'library_html5_webgl.js',
         'library_openal.js',
-        'library_sdl.js',
         'library_glut.js',
         'library_xlib.js',
         'library_egl.js',
-        'library_glfw.js',
         'library_uuid.js',
         'library_glew.js',
         'library_idbstore.js',
         'library_async.js',
       ]);
+      if (USE_SDL != 2) {
+        libraries.push('library_sdl.js');
+      }
     } else {
       if (ASYNCIFY) {
         libraries.push('library_async.js');
@@ -131,6 +145,10 @@ global.LibraryManager = {
       }
     }
 
+    if (USE_GLFW) {
+      libraries.push('library_glfw.js');
+    }
+
     if (LZ4) {
       libraries.push('library_lz4.js');
     }
@@ -193,7 +211,6 @@ global.LibraryManager = {
           printErr('processing system library: ' + filename);
         }
       }
-      const src = read(filename);
       let origLibrary = undefined;
       let processed = undefined;
       // When we parse user libraries also set `__user` attribute
@@ -201,61 +218,49 @@ global.LibraryManager = {
       if (isUserLibrary) {
         origLibrary = this.library;
         this.library = new Proxy(this.library, {
-          set: (target, prop, value) => {
+          set(target, prop, value) {
             target[prop] = value;
-            if (!isJsLibraryConfigIdentifier(prop)) {
+            if (!isDecorator(prop)) {
               target[prop + '__user'] = true;
             }
             return true;
           },
         });
       }
+      currentFile = filename;
       try {
-        processed = processMacros(preprocess(src, filename));
-        eval(processed);
+        processed = processMacros(preprocess(filename));
+        vm.runInThisContext(processed, { filename: filename.replace(/\.\w+$/, '.preprocessed$&') });
       } catch (e) {
-        const details = [e, e.lineNumber ? `line number: ${e.lineNumber}` : ''];
+        error(`failure to execute js library "${filename}":`);
         if (VERBOSE) {
-          details.push((e.stack || '').toString().replace('Object.<anonymous>', filename));
-        }
-        if (processed) {
-          error(`failure to execute js library "${filename}": ${details}`);
-          if (VERBOSE) {
+          const orig = read(filename);
+          if (processed) {
             error(`preprocessed source (you can run a js engine on this to get a clearer error message sometimes):\n=============\n${processed}\n=============`);
           } else {
-            error('use -sVERBOSE to see more details');
+            error(`original source:\n=============\n${orig}\n=============`);
           }
         } else {
-          error(`failure to process js library "${filename}": ${details}`);
-          if (VERBOSE) {
-            error(`original source:\n=============\n${src}\n=============`);
-          } else {
-            error('use -sVERBOSE to see more details');
-          }
+          error('use -sVERBOSE to see more details');
         }
         throw e;
       } finally {
+        currentFile = null;
         if (origLibrary) {
           this.library = origLibrary;
         }
       }
     }
-
-    for (const ident in this.library) {
-      if (isJsLibraryConfigIdentifier(ident)) {
-        const index = ident.lastIndexOf('__');
-        const basename = ident.slice(0, index);
-        if (!(basename in this.library)) {
-          error(`Missing library element '${basename}' for library config '${ident}'`);
-        }
-      }
-    }
   },
 };
 
 if (!BOOTSTRAPPING_STRUCT_INFO) {
+  let structInfoFile = 'generated_struct_info32.json';
+  if (MEMORY64) {
+    structInfoFile = 'generated_struct_info64.json'
+  }
   // Load struct and define information.
-  const temp = JSON.parse(read(STRUCT_INFO));
+  const temp = JSON.parse(read(structInfoFile));
   C_STRUCTS = temp.structs;
   C_DEFINES = temp.defines;
 } else {
@@ -263,49 +268,66 @@ if (!BOOTSTRAPPING_STRUCT_INFO) {
   C_DEFINES = {};
 }
 
-// Safe way to access a C define. We check that we don't add library functions with missing defines.
-function cDefine(key) {
-  if (key in C_DEFINES) return C_DEFINES[key];
-  throw new Error(`Missing C define ${key}! If you just added it to struct_info.json, you need to ./emcc --clear-cache`);
-}
+// Use proxy objects for C_DEFINES and C_STRUCTS so that we can give useful
+// error messages.
+C_STRUCTS = new Proxy(C_STRUCTS, {
+  get(target, prop, receiver) {
+    if (!(prop in target)) {
+      throw new Error(`Missing C struct ${prop}! If you just added it to struct_info.json, you need to run ./tools/gen_struct_info.py`);
+    }
+    return target[prop]
+  }
+});
 
-function isFSPrefixed(name) {
-  return name.length > 3 && name[0] === 'F' && name[1] === 'S' && name[2] === '_';
+cDefs = C_DEFINES = new Proxy(C_DEFINES, {
+  get(target, prop, receiver) {
+    if (!(prop in target)) {
+      throw new Error(`Missing C define ${prop}! If you just added it to struct_info.json, you need to run ./tools/gen_struct_info.py`);
+    }
+    return target[prop]
+  }
+});
+
+// Legacy function that existed solely to give error message.  These are now
+// provided by the cDefs proxy object above.
+function cDefine(key) {
+  return cDefs[key];
 }
 
 function isInternalSymbol(ident) {
   return ident + '__internal' in LibraryManager.library;
 }
 
+function getUnusedLibarySymbols() {
+  const librarySymbolSet = new Set(librarySymbols);
+  const missingSyms = new Set();
+  for (const [ident, value] of Object.entries(LibraryManager.library)) {
+    if (typeof value === 'function' || typeof value === 'number') {
+      if (isJsOnlySymbol(ident) && !isDecorator(ident) && !isInternalSymbol(ident)) {
+        const name = ident.substr(1);
+        if (!librarySymbolSet.has(name)) {
+          missingSyms.add(name);
+        }
+      }
+    }
+  }
+  return missingSyms;
+}
+
 // When running with ASSERTIONS enabled we create stubs for each library
 // function that that was not included in the build.  This gives useful errors
 // when library dependencies are missing from `__deps` or depended on without
 // being added to DEFAULT_LIBRARY_FUNCS_TO_INCLUDE
 // TODO(sbc): These errors could potentially be generated at build time via
 // some kind of acorn pass that searched for uses of these missing symbols.
-function addMissingLibraryStubs() {
-  if (!ASSERTIONS) return '';
+function addMissingLibraryStubs(unusedLibSymbols) {
   let rtn = '';
-  const librarySymbolSet = new Set(librarySymbols);
-  const missingSyms = [];
-  for (const ident in LibraryManager.library) {
-    if (typeof LibraryManager.library[ident] === 'function' || typeof LibraryManager.library[ident] === 'number') {
-      if (ident[0] === '$' && !isJsLibraryConfigIdentifier(ident) && !isInternalSymbol(ident)) {
-        const name = ident.substr(1);
-        if (!librarySymbolSet.has(name)) {
-          missingSyms.push(name);
-        }
-      }
-    }
-  }
-  if (missingSyms.length) {
-    rtn += 'var missingLibrarySymbols = [\n';
-    for (const sym of missingSyms) {
-      rtn += `  '${sym}',\n`;
-    }
-    rtn += '];\n';
-    rtn += 'missingLibrarySymbols.forEach(missingLibrarySymbol)\n';
+  rtn += 'var missingLibrarySymbols = [\n';
+  for (const sym of unusedLibSymbols) {
+    rtn += `  '${sym}',\n`;
   }
+  rtn += '];\n';
+  rtn += 'missingLibrarySymbols.forEach(missingLibrarySymbol)\n';
   return rtn;
 }
 
@@ -327,24 +349,19 @@ function exportRuntime() {
     if (EXPORTED_RUNTIME_METHODS_SET.has(name)) {
       let exported = name;
       // the exported name may differ from the internal name
-      if (isFSPrefixed(exported)) {
+      if (exported.startsWith('FS_')) {
         // this is a filesystem value, FS.x exported as FS_x
         exported = 'FS.' + exported.substr(3);
       } else if (legacyRuntimeElements.has(exported)) {
         exported = legacyRuntimeElements.get(exported);
       }
-      return `Module["${name}"] = ${exported};`;
+      return `Module['${name}'] = ${exported};`;
     }
   }
 
   // All possible runtime elements that can be exported
   let runtimeElements = [
     'run',
-    'UTF8ArrayToString',
-    'UTF8ToString',
-    'stringToUTF8Array',
-    'stringToUTF8',
-    'lengthBytesUTF8',
     'addOnPreRun',
     'addOnInit',
     'addOnPreMain',
@@ -355,30 +372,25 @@ function exportRuntime() {
     'FS_createFolder',
     'FS_createPath',
     'FS_createDataFile',
-    'FS_createPreloadedFile',
     'FS_createLazyFile',
     'FS_createLink',
     'FS_createDevice',
     'FS_unlink',
-    'getLEB',
-    'getFunctionTables',
-    'alignFunctionTables',
-    'registerFunctions',
-    'prettyPrint',
-    'getCompilerSetting',
     'out',
     'err',
     'callMain',
     'abort',
     'keepRuntimeAlive',
     'wasmMemory',
+    'wasmTable',
+    'wasmExports',
   ];
 
   // These are actually native wasm functions these days but we allow exporting
   // them via EXPORTED_RUNTIME_METHODS for backwards compat.
   runtimeElements = runtimeElements.concat(WASM_SYSTEM_EXPORTS);
 
-  if (USE_PTHREADS && ALLOW_MEMORY_GROWTH) {
+  if (PTHREADS && ALLOW_MEMORY_GROWTH) {
     runtimeElements = runtimeElements.concat([
       'GROWABLE_HEAP_I8',
       'GROWABLE_HEAP_U8',
@@ -408,6 +420,14 @@ function exportRuntime() {
     runtimeElements.push('tryParseAsDataURI');
   }
 
+  if (RETAIN_COMPILER_SETTINGS) {
+    runtimeElements.push('getCompilerSetting')
+  }
+
+  if (RUNTIME_DEBUG) {
+    runtimeElements.push('prettyPrint')
+  }
+
   // dynCall_* methods are not hardcoded here, as they
   // depend on the file being compiled. check for them
   // and add them.
@@ -430,43 +450,49 @@ function exportRuntime() {
 
   // Add JS library elements such as FS, GL, ENV, etc. These are prefixed with
   // '$ which indicates they are JS methods.
-  const runtimeElementsSet = new Set(runtimeElements);
-  for (const ident in LibraryManager.library) {
-    if (ident[0] === '$' && !isJsLibraryConfigIdentifier(ident) && !isInternalSymbol(ident)) {
+  let runtimeElementsSet = new Set(runtimeElements);
+  for (const ident of Object.keys(LibraryManager.library)) {
+    if (isJsOnlySymbol(ident) && !isDecorator(ident) && !isInternalSymbol(ident)) {
       const jsname = ident.substr(1);
       assert(!runtimeElementsSet.has(jsname), 'runtimeElements contains library symbol: ' + ident);
       runtimeElements.push(jsname);
     }
   }
 
-  let unexportedStubs = '';
-  if (ASSERTIONS) {
-    // check all exported things exist, warn about typos
-    const runtimeElementsSet = new Set(runtimeElements);
-    for (const name of EXPORTED_RUNTIME_METHODS_SET) {
-      if (!runtimeElementsSet.has(name)) {
-        warn(`invalid item in EXPORTED_RUNTIME_METHODS: ${name}`);
-      }
+  // check all exported things exist, warn about typos
+  runtimeElementsSet = new Set(runtimeElements);
+  for (const name of EXPORTED_RUNTIME_METHODS_SET) {
+    if (!runtimeElementsSet.has(name)) {
+      warn(`invalid item in EXPORTED_RUNTIME_METHODS: ${name}`);
+    }
+  }
+
+  const exports = runtimeElements.map(maybeExport);
+  const results = exports.filter((name) => name);
+
+  if (ASSERTIONS && !EXPORT_ALL) {
+    const unusedLibSymbols = getUnusedLibarySymbols();
+    if (unusedLibSymbols.size) {
+      results.push(addMissingLibraryStubs(unusedLibSymbols));
     }
 
     const unexported = [];
     for (const name of runtimeElements) {
-      if (!EXPORTED_RUNTIME_METHODS_SET.has(name)) {
+      if (!EXPORTED_RUNTIME_METHODS_SET.has(name) && !unusedLibSymbols.has(name)) {
         unexported.push(name);
       }
     }
 
-    if (unexported.length) {
-      unexportedStubs += 'var unexportedRuntimeSymbols = [\n';
+    if (unexported.length || unusedLibSymbols.size) {
+      let unexportedStubs = 'var unexportedSymbols = [\n';
       for (const sym of unexported) {
         unexportedStubs += `  '${sym}',\n`;
       }
       unexportedStubs += '];\n';
-      unexportedStubs += 'unexportedRuntimeSymbols.forEach(unexportedRuntimeSymbol);\n';
+      unexportedStubs += 'unexportedSymbols.forEach(unexportedRuntimeSymbol);\n';
+      results.push(unexportedStubs);
     }
   }
 
-  let exports = runtimeElements.map((name) => maybeExport(name));
-  exports = exports.filter((name) => name);
-  return exports.join('\n') + '\n' + unexportedStubs + addMissingLibraryStubs();
+  return results.join('\n') + '\n';
 }
