@@ -16,8 +16,8 @@
 // underscore.
 var WASM_EXPORTS = [];
 
-// Similar to above but only includes the functions symbols.
-var WASM_FUNCTION_EXPORTS = [];
+// Similar to above but only includes the global/data symbols.
+var WASM_GLOBAL_EXPORTS = [];
 
 // An array of all symbols exported from all the side modules specified on the
 // command line.
@@ -97,7 +97,7 @@ var FETCH_WORKER_FILE = '';
 
 var WASI_MODULE_NAME = "wasi_snapshot_preview1";
 
-// List of JS libraries explictly linked against.  This includes JS system
+// List of JS libraries explicitly linked against.  This includes JS system
 // libraries (specified via -lfoo or -lfoo.js) in addition to user libraries
 // passed via `--js-library`.  It does not include implicitly linked libraries
 // added by the JS compiler.
@@ -137,10 +137,16 @@ var PTHREAD_WORKER_FILE = '';
 // name of the file containing the Wasm Worker *.ww.js, if relevant
 var WASM_WORKER_FILE = '';
 
+// name of the file containing the Audio Worklet *.aw.js, if relevant
+var AUDIO_WORKLET_FILE = '';
+
 // Base URL the source mapfile, if relevant
 var SOURCE_MAP_BASE = '';
 
-var MEM_INIT_IN_WASM = false;
+// When this is false we use an external memory init file
+// See --memory-init-file.  When not using wasm2js this flag is ignored, and
+// this setting will always be true.
+var MEM_INIT_IN_WASM = true;
 
 // If set to 1, src/base64Utils.js will be included in the bundle.
 // This is set internally when needed (SINGLE_FILE)
@@ -160,12 +166,18 @@ var MINIFY_WASM_IMPORTS_AND_EXPORTS = false;
 // Whether to minify imported module names.
 var MINIFY_WASM_IMPORTED_MODULES = false;
 
-// Whether to minify functions exported from Asm.js/Wasm module.
-var MINIFY_ASMJS_EXPORT_NAMES = true;
+// Whether to minify exports from the Wasm module.
+var MINIFY_WASM_EXPORT_NAMES = true;
 
 // Internal: represents a browser version that is not supported at all.
 var TARGET_NOT_SUPPORTED = 0x7FFFFFFF;
 
+// Used to track whether target environment supports the 'globalThis' attribute.
+var SUPPORTS_GLOBALTHIS = false;
+
+// Used to track whether target environment supports the 'Promise.any'.
+var SUPPORTS_PROMISE_ANY = false;
+
 // Wasm backend symbols that are considered system symbols and don't
 // have the normal C symbol name mangled applied (== prefix with an underscore)
 // (Also implicily on this list is any function that starts with string "dynCall_")
@@ -174,8 +186,12 @@ var WASM_SYSTEM_EXPORTS = ['stackAlloc', 'stackSave', 'stackRestore', 'getTempRe
 // Internal: value of -flto argument (either full or thin)
 var LTO = 0;
 
-// Whether we may be accessing the address 2GB or higher. If so then we need
-// to be using unsigned pointers in JS.
+// Whether we may be accessing the address 2GB or higher. If so, then we need
+// to interpret incoming i32 pointers as unsigned.
+//
+// This setting does not apply (and is never set to true) under MEMORY64, since
+// in that case we get 64-bit pointers coming through to JS (converting them to
+// i53 in most cases).
 var CAN_ADDRESS_2GB = false;
 
 // Whether to emit DWARF in a separate wasm file on the side (this is not called
@@ -189,11 +205,6 @@ var SEPARATE_DWARF = false;
 // New WebAssembly exception handling
 var WASM_EXCEPTIONS = false;
 
-// Used internally when running the JS compiler simply to generate list of all
-// JS symbols. This is used by LLD_REPORT_UNDEFINED to generate a list of all
-// JS library symbols.
-var ONLY_CALC_JS_SYMBOLS = false;
-
 // Set to true if the program has a main function.  By default this is
 // enabled, but if `--no-entry` is passed, or if `_main` is not part of
 // EXPORTED_FUNCTIONS then this gets set to 0.
@@ -203,9 +214,6 @@ var EXPECT_MAIN = true;
 // MODULARIZE, and returned from the factory function.
 var EXPORT_READY_PROMISE = true;
 
-// struct_info that is either generated or cached
-var STRUCT_INFO = '';
-
 // If true, building against Emscripten's wasm heap memory profiler.
 var MEMORYPROFILER = false;
 
@@ -231,7 +239,7 @@ var HAS_MAIN = false;
 var LINK_AS_CXX = false;
 
 // Set when some minimum browser version triggers doesn't support the
-// minimum set of ES6 featurs.  This triggers transpilation to ES5
+// minimum set of ES6 features.  This triggers transpilation to ES5
 // using closure compiler.
 var TRANSPILE_TO_ES5 = false;
 
@@ -246,3 +254,14 @@ var ALL_INCOMING_MODULE_JS_API = [];
 var WEAK_IMPORTS = [];
 
 var STACK_FIRST = false;
+
+var HAVE_EM_ASM = true;
+
+var PRE_JS_FILES = [];
+
+var POST_JS_FILES = [];
+
+// Set when -pthread / -sPTHREADS is passed
+var PTHREADS = false;
+
+var BULK_MEMORY = false;
