@@ -28,23 +28,15 @@ if (ENVIRONMENT_IS_NODE) {
 
   Object.assign(global, {
     self: global,
-    require: require,
-    Module: Module,
+    require,
+    Module,
     location: {
       href: __filename
     },
     Worker: nodeWorkerThreads.Worker,
-    importScripts: function(f) {
-      (0, eval)(fs.readFileSync(f, 'utf8') + '//# sourceURL=' + f);
-    },
-    postMessage: function(msg) {
-      parentPort.postMessage(msg);
-    },
-    performance: global.performance || {
-      now: function() {
-        return Date.now();
-      }
-    },
+    importScripts: (f) => (0, eval)(fs.readFileSync(f, 'utf8') + '//# sourceURL=' + f),
+    postMessage: (msg) => parentPort.postMessage(msg),
+    performance: global.performance || { now: Date.now },
   });
 }
 #endif // ENVIRONMENT_MAY_BE_NODE
@@ -71,7 +63,7 @@ function threadPrintErr() {
 }
 function threadAlert() {
   var text = Array.prototype.slice.call(arguments).join(' ');
-  postMessage({cmd: 'alert', text: text, threadId: Module['_pthread_self']()});
+  postMessage({cmd: 'alert', text, threadId: Module['_pthread_self']()});
 }
 #if ASSERTIONS
 // We don't need out() for now, but may need to add it if we want to use it
@@ -160,8 +152,11 @@ function handleMessage(e) {
       // Use `const` here to ensure that the variable is scoped only to
       // that iteration, allowing safe reference from a closure.
       for (const handler of e.data.handlers) {
-        Module[handler] = function() {
-          postMessage({ cmd: 'callHandler', handler, args: [...arguments] });
+        Module[handler] = (...args) => {
+#if RUNTIME_DEBUG
+          dbg(`calling handler on main thread: ${handler}`);
+#endif
+          postMessage({ cmd: 'callHandler', handler, args: args });
         }
       }
 
@@ -191,7 +186,7 @@ function handleMessage(e) {
       if (typeof e.data.urlOrBlob == 'string') {
 #if TRUSTED_TYPES
         if (typeof self.trustedTypes != 'undefined' && self.trustedTypes.createPolicy) {
-          var p = self.trustedTypes.createPolicy('emscripten#workerPolicy3', { createScriptURL: function(ignored) { return e.data.urlOrBlob } });
+          var p = self.trustedTypes.createPolicy('emscripten#workerPolicy3', { createScriptURL: (ignored) => e.data.urlOrBlob });
           importScripts(p.createScriptURL('ignored'));
         } else
 #endif
@@ -200,7 +195,7 @@ function handleMessage(e) {
         var objectUrl = URL.createObjectURL(e.data.urlOrBlob);
 #if TRUSTED_TYPES
         if (typeof self.trustedTypes != 'undefined' && self.trustedTypes.createPolicy) {
-          var p = self.trustedTypes.createPolicy('emscripten#workerPolicy3', { createScriptURL: function(ignored) { return objectUrl } });
+          var p = self.trustedTypes.createPolicy('emscripten#workerPolicy3', { createScriptURL: (ignored) => objectUrl });
           importScripts(p.createScriptURL('ignored'));
         } else
 #endif
