@@ -128,6 +128,12 @@ namespace wgpu {
         return (static_cast<Integral>(value) & (static_cast<Integral>(value) - 1)) == 0;
     }
 
+    namespace detail {
+        constexpr size_t ConstexprMax(size_t a, size_t b) {
+            return a > b ? a : b;
+        }
+    }  // namespace detail
+
     static constexpr uint32_t kArrayLayerCountUndefined = WGPU_ARRAY_LAYER_COUNT_UNDEFINED;
     static constexpr uint32_t kCopyStrideUndefined = WGPU_COPY_STRIDE_UNDEFINED;
     static constexpr uint32_t kLimitU32Undefined = WGPU_LIMIT_U32_UNDEFINED;
@@ -150,14 +156,15 @@ namespace wgpu {
     };
 
     enum class BackendType : uint32_t {
-        Null = 0x00000000,
-        WebGPU = 0x00000001,
-        D3D11 = 0x00000002,
-        D3D12 = 0x00000003,
-        Metal = 0x00000004,
-        Vulkan = 0x00000005,
-        OpenGL = 0x00000006,
-        OpenGLES = 0x00000007,
+        Undefined = 0x00000000,
+        Null = 0x00000001,
+        WebGPU = 0x00000002,
+        D3D11 = 0x00000003,
+        D3D12 = 0x00000004,
+        Metal = 0x00000005,
+        Vulkan = 0x00000006,
+        OpenGL = 0x00000007,
+        OpenGLES = 0x00000008,
     };
 
     enum class BlendFactor : uint32_t {
@@ -193,11 +200,20 @@ namespace wgpu {
 
     enum class BufferMapAsyncStatus : uint32_t {
         Success = 0x00000000,
-        Error = 0x00000001,
+        ValidationError = 0x00000001,
         Unknown = 0x00000002,
         DeviceLost = 0x00000003,
         DestroyedBeforeCallback = 0x00000004,
         UnmappedBeforeCallback = 0x00000005,
+        MappingAlreadyPending = 0x00000006,
+        OffsetOutOfRange = 0x00000007,
+        SizeOutOfRange = 0x00000008,
+    };
+
+    enum class BufferMapState : uint32_t {
+        Unmapped = 0x00000000,
+        Pending = 0x00000001,
+        Mapped = 0x00000002,
     };
 
     enum class CompareFunction : uint32_t {
@@ -225,12 +241,18 @@ namespace wgpu {
         Info = 0x00000002,
     };
 
+    enum class ComputePassTimestampLocation : uint32_t {
+        Beginning = 0x00000000,
+        End = 0x00000001,
+    };
+
     enum class CreatePipelineAsyncStatus : uint32_t {
         Success = 0x00000000,
-        Error = 0x00000001,
-        DeviceLost = 0x00000002,
-        DeviceDestroyed = 0x00000003,
-        Unknown = 0x00000004,
+        ValidationError = 0x00000001,
+        InternalError = 0x00000002,
+        DeviceLost = 0x00000003,
+        DeviceDestroyed = 0x00000004,
+        Unknown = 0x00000005,
     };
 
     enum class CullMode : uint32_t {
@@ -247,28 +269,32 @@ namespace wgpu {
     enum class ErrorFilter : uint32_t {
         Validation = 0x00000000,
         OutOfMemory = 0x00000001,
+        Internal = 0x00000002,
     };
 
     enum class ErrorType : uint32_t {
         NoError = 0x00000000,
         Validation = 0x00000001,
         OutOfMemory = 0x00000002,
-        Unknown = 0x00000003,
-        DeviceLost = 0x00000004,
+        Internal = 0x00000003,
+        Unknown = 0x00000004,
+        DeviceLost = 0x00000005,
     };
 
     enum class FeatureName : uint32_t {
         Undefined = 0x00000000,
         DepthClipControl = 0x00000001,
-        Depth24UnormStencil8 = 0x00000002,
-        Depth32FloatStencil8 = 0x00000003,
-        TimestampQuery = 0x00000004,
-        PipelineStatisticsQuery = 0x00000005,
-        TextureCompressionBC = 0x00000006,
-        TextureCompressionETC2 = 0x00000007,
-        TextureCompressionASTC = 0x00000008,
-        IndirectFirstInstance = 0x00000009,
-        DepthClamping = 0x000003E8,
+        Depth32FloatStencil8 = 0x00000002,
+        TimestampQuery = 0x00000003,
+        PipelineStatisticsQuery = 0x00000004,
+        TextureCompressionBC = 0x00000005,
+        TextureCompressionETC2 = 0x00000006,
+        TextureCompressionASTC = 0x00000007,
+        IndirectFirstInstance = 0x00000008,
+        ShaderF16 = 0x00000009,
+        RG11B10UfloatRenderable = 0x0000000A,
+        BGRA8UnormStorage = 0x0000000B,
+        Float32Filterable = 0x0000000C,
     };
 
     enum class FilterMode : uint32_t {
@@ -293,6 +319,11 @@ namespace wgpu {
         Load = 0x00000002,
     };
 
+    enum class MipmapFilterMode : uint32_t {
+        Nearest = 0x00000000,
+        Linear = 0x00000001,
+    };
+
     enum class PipelineStatisticName : uint32_t {
         VertexShaderInvocations = 0x00000000,
         ClipperInvocations = 0x00000001,
@@ -307,11 +338,6 @@ namespace wgpu {
         HighPerformance = 0x00000002,
     };
 
-    enum class PredefinedColorSpace : uint32_t {
-        Undefined = 0x00000000,
-        Srgb = 0x00000001,
-    };
-
     enum class PresentMode : uint32_t {
         Immediate = 0x00000000,
         Mailbox = 0x00000001,
@@ -339,6 +365,11 @@ namespace wgpu {
         DeviceLost = 0x00000003,
     };
 
+    enum class RenderPassTimestampLocation : uint32_t {
+        Beginning = 0x00000000,
+        End = 0x00000001,
+    };
+
     enum class RequestAdapterStatus : uint32_t {
         Success = 0x00000000,
         Unavailable = 0x00000001,
@@ -358,7 +389,7 @@ namespace wgpu {
         ShaderModuleSPIRVDescriptor = 0x00000005,
         ShaderModuleWGSLDescriptor = 0x00000006,
         PrimitiveDepthClipControl = 0x00000007,
-        PrimitiveDepthClampingState = 0x000003E9,
+        RenderPassDescriptorMaxDrawCount = 0x0000000F,
     };
 
     enum class SamplerBindingType : uint32_t {
@@ -396,13 +427,6 @@ namespace wgpu {
         DepthOnly = 0x00000002,
     };
 
-    enum class TextureComponentType : uint32_t {
-        Float = 0x00000000,
-        Sint = 0x00000001,
-        Uint = 0x00000002,
-        DepthComparison = 0x00000003,
-    };
-
     enum class TextureDimension : uint32_t {
         e1D = 0x00000000,
         e2D = 0x00000001,
@@ -451,61 +475,60 @@ namespace wgpu {
         Depth16Unorm = 0x00000026,
         Depth24Plus = 0x00000027,
         Depth24PlusStencil8 = 0x00000028,
-        Depth24UnormStencil8 = 0x00000029,
-        Depth32Float = 0x0000002A,
-        Depth32FloatStencil8 = 0x0000002B,
-        BC1RGBAUnorm = 0x0000002C,
-        BC1RGBAUnormSrgb = 0x0000002D,
-        BC2RGBAUnorm = 0x0000002E,
-        BC2RGBAUnormSrgb = 0x0000002F,
-        BC3RGBAUnorm = 0x00000030,
-        BC3RGBAUnormSrgb = 0x00000031,
-        BC4RUnorm = 0x00000032,
-        BC4RSnorm = 0x00000033,
-        BC5RGUnorm = 0x00000034,
-        BC5RGSnorm = 0x00000035,
-        BC6HRGBUfloat = 0x00000036,
-        BC6HRGBFloat = 0x00000037,
-        BC7RGBAUnorm = 0x00000038,
-        BC7RGBAUnormSrgb = 0x00000039,
-        ETC2RGB8Unorm = 0x0000003A,
-        ETC2RGB8UnormSrgb = 0x0000003B,
-        ETC2RGB8A1Unorm = 0x0000003C,
-        ETC2RGB8A1UnormSrgb = 0x0000003D,
-        ETC2RGBA8Unorm = 0x0000003E,
-        ETC2RGBA8UnormSrgb = 0x0000003F,
-        EACR11Unorm = 0x00000040,
-        EACR11Snorm = 0x00000041,
-        EACRG11Unorm = 0x00000042,
-        EACRG11Snorm = 0x00000043,
-        ASTC4x4Unorm = 0x00000044,
-        ASTC4x4UnormSrgb = 0x00000045,
-        ASTC5x4Unorm = 0x00000046,
-        ASTC5x4UnormSrgb = 0x00000047,
-        ASTC5x5Unorm = 0x00000048,
-        ASTC5x5UnormSrgb = 0x00000049,
-        ASTC6x5Unorm = 0x0000004A,
-        ASTC6x5UnormSrgb = 0x0000004B,
-        ASTC6x6Unorm = 0x0000004C,
-        ASTC6x6UnormSrgb = 0x0000004D,
-        ASTC8x5Unorm = 0x0000004E,
-        ASTC8x5UnormSrgb = 0x0000004F,
-        ASTC8x6Unorm = 0x00000050,
-        ASTC8x6UnormSrgb = 0x00000051,
-        ASTC8x8Unorm = 0x00000052,
-        ASTC8x8UnormSrgb = 0x00000053,
-        ASTC10x5Unorm = 0x00000054,
-        ASTC10x5UnormSrgb = 0x00000055,
-        ASTC10x6Unorm = 0x00000056,
-        ASTC10x6UnormSrgb = 0x00000057,
-        ASTC10x8Unorm = 0x00000058,
-        ASTC10x8UnormSrgb = 0x00000059,
-        ASTC10x10Unorm = 0x0000005A,
-        ASTC10x10UnormSrgb = 0x0000005B,
-        ASTC12x10Unorm = 0x0000005C,
-        ASTC12x10UnormSrgb = 0x0000005D,
-        ASTC12x12Unorm = 0x0000005E,
-        ASTC12x12UnormSrgb = 0x0000005F,
+        Depth32Float = 0x00000029,
+        Depth32FloatStencil8 = 0x0000002A,
+        BC1RGBAUnorm = 0x0000002B,
+        BC1RGBAUnormSrgb = 0x0000002C,
+        BC2RGBAUnorm = 0x0000002D,
+        BC2RGBAUnormSrgb = 0x0000002E,
+        BC3RGBAUnorm = 0x0000002F,
+        BC3RGBAUnormSrgb = 0x00000030,
+        BC4RUnorm = 0x00000031,
+        BC4RSnorm = 0x00000032,
+        BC5RGUnorm = 0x00000033,
+        BC5RGSnorm = 0x00000034,
+        BC6HRGBUfloat = 0x00000035,
+        BC6HRGBFloat = 0x00000036,
+        BC7RGBAUnorm = 0x00000037,
+        BC7RGBAUnormSrgb = 0x00000038,
+        ETC2RGB8Unorm = 0x00000039,
+        ETC2RGB8UnormSrgb = 0x0000003A,
+        ETC2RGB8A1Unorm = 0x0000003B,
+        ETC2RGB8A1UnormSrgb = 0x0000003C,
+        ETC2RGBA8Unorm = 0x0000003D,
+        ETC2RGBA8UnormSrgb = 0x0000003E,
+        EACR11Unorm = 0x0000003F,
+        EACR11Snorm = 0x00000040,
+        EACRG11Unorm = 0x00000041,
+        EACRG11Snorm = 0x00000042,
+        ASTC4x4Unorm = 0x00000043,
+        ASTC4x4UnormSrgb = 0x00000044,
+        ASTC5x4Unorm = 0x00000045,
+        ASTC5x4UnormSrgb = 0x00000046,
+        ASTC5x5Unorm = 0x00000047,
+        ASTC5x5UnormSrgb = 0x00000048,
+        ASTC6x5Unorm = 0x00000049,
+        ASTC6x5UnormSrgb = 0x0000004A,
+        ASTC6x6Unorm = 0x0000004B,
+        ASTC6x6UnormSrgb = 0x0000004C,
+        ASTC8x5Unorm = 0x0000004D,
+        ASTC8x5UnormSrgb = 0x0000004E,
+        ASTC8x6Unorm = 0x0000004F,
+        ASTC8x6UnormSrgb = 0x00000050,
+        ASTC8x8Unorm = 0x00000051,
+        ASTC8x8UnormSrgb = 0x00000052,
+        ASTC10x5Unorm = 0x00000053,
+        ASTC10x5UnormSrgb = 0x00000054,
+        ASTC10x6Unorm = 0x00000055,
+        ASTC10x6UnormSrgb = 0x00000056,
+        ASTC10x8Unorm = 0x00000057,
+        ASTC10x8UnormSrgb = 0x00000058,
+        ASTC10x10Unorm = 0x00000059,
+        ASTC10x10UnormSrgb = 0x0000005A,
+        ASTC12x10Unorm = 0x0000005B,
+        ASTC12x10UnormSrgb = 0x0000005C,
+        ASTC12x12Unorm = 0x0000005D,
+        ASTC12x12UnormSrgb = 0x0000005E,
     };
 
     enum class TextureSampleType : uint32_t {
@@ -564,6 +587,7 @@ namespace wgpu {
     enum class VertexStepMode : uint32_t {
         Vertex = 0x00000000,
         Instance = 0x00000001,
+        VertexBufferNotUsed = 0x00000002,
     };
 
 
@@ -657,7 +681,7 @@ namespace wgpu {
     struct CommandBufferDescriptor;
     struct CommandEncoderDescriptor;
     struct CompilationMessage;
-    struct ComputePassDescriptor;
+    struct ComputePassTimestampWrite;
     struct ConstantEntry;
     struct Extent3D;
     struct InstanceDescriptor;
@@ -665,13 +689,15 @@ namespace wgpu {
     struct MultisampleState;
     struct Origin3D;
     struct PipelineLayoutDescriptor;
-    struct PrimitiveDepthClampingState;
     struct PrimitiveDepthClipControl;
     struct PrimitiveState;
     struct QuerySetDescriptor;
+    struct QueueDescriptor;
     struct RenderBundleDescriptor;
     struct RenderBundleEncoderDescriptor;
     struct RenderPassDepthStencilAttachment;
+    struct RenderPassDescriptorMaxDrawCount;
+    struct RenderPassTimestampWrite;
     struct RequestAdapterOptions;
     struct SamplerBindingLayout;
     struct SamplerDescriptor;
@@ -691,6 +717,7 @@ namespace wgpu {
     struct BindGroupLayoutEntry;
     struct BlendState;
     struct CompilationInfo;
+    struct ComputePassDescriptor;
     struct DepthStencilState;
     struct ImageCopyBuffer;
     struct ImageCopyTexture;
@@ -769,11 +796,17 @@ namespace wgpu {
         CType Get() const {
             return mHandle;
         }
+        // TODO(dawn:1639) Deprecate Release after uses have been removed.
         CType Release() {
             CType result = mHandle;
             mHandle = 0;
             return result;
         }
+        CType MoveToCHandle() {
+            CType result = mHandle;
+            mHandle = 0;
+            return result;
+        }
         static Derived Acquire(CType handle) {
             Derived result;
             result.mHandle = handle;
@@ -791,6 +824,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        size_t EnumerateFeatures(FeatureName * features) const;
         bool GetLimits(SupportedLimits * limits) const;
         void GetProperties(AdapterProperties * properties) const;
         bool HasFeature(FeatureName feature) const;
@@ -807,6 +841,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<BindGroup, WGPUBindGroup>;
@@ -819,6 +854,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<BindGroupLayout, WGPUBindGroupLayout>;
@@ -832,9 +868,13 @@ namespace wgpu {
         using ObjectBase::operator=;
 
         void Destroy() const;
-        void const * GetConstMappedRange(size_t offset = 0, size_t size = 0) const;
-        void * GetMappedRange(size_t offset = 0, size_t size = 0) const;
+        void const * GetConstMappedRange(size_t offset = 0, size_t size = WGPU_WHOLE_MAP_SIZE) const;
+        BufferMapState GetMapState() const;
+        void * GetMappedRange(size_t offset = 0, size_t size = WGPU_WHOLE_MAP_SIZE) const;
+        uint64_t GetSize() const;
+        BufferUsage GetUsage() const;
         void MapAsync(MapMode mode, size_t offset, size_t size, BufferMapCallback callback, void * userdata) const;
+        void SetLabel(char const * label) const;
         void Unmap() const;
 
       private:
@@ -848,6 +888,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<CommandBuffer, WGPUCommandBuffer>;
@@ -872,6 +913,7 @@ namespace wgpu {
         void PopDebugGroup() const;
         void PushDebugGroup(char const * groupLabel) const;
         void ResolveQuerySet(QuerySet const& querySet, uint32_t firstQuery, uint32_t queryCount, Buffer const& destination, uint64_t destinationOffset) const;
+        void SetLabel(char const * label) const;
         void WriteTimestamp(QuerySet const& querySet, uint32_t queryIndex) const;
 
       private:
@@ -886,14 +928,15 @@ namespace wgpu {
         using ObjectBase::operator=;
 
         void BeginPipelineStatisticsQuery(QuerySet const& querySet, uint32_t queryIndex) const;
-        void Dispatch(uint32_t workgroupCountX, uint32_t workgroupCountY = 1, uint32_t workgroupCountZ = 1) const;
-        void DispatchIndirect(Buffer const& indirectBuffer, uint64_t indirectOffset) const;
+        void DispatchWorkgroups(uint32_t workgroupCountX, uint32_t workgroupCountY = 1, uint32_t workgroupCountZ = 1) const;
+        void DispatchWorkgroupsIndirect(Buffer const& indirectBuffer, uint64_t indirectOffset) const;
         void End() const;
         void EndPipelineStatisticsQuery() const;
         void InsertDebugMarker(char const * markerLabel) const;
         void PopDebugGroup() const;
         void PushDebugGroup(char const * groupLabel) const;
-        void SetBindGroup(uint32_t groupIndex, BindGroup const& group, uint32_t dynamicOffsetCount = 0, uint32_t const * dynamicOffsets = nullptr) const;
+        void SetBindGroup(uint32_t groupIndex, BindGroup const& group, size_t dynamicOffsetCount = 0, uint32_t const * dynamicOffsets = nullptr) const;
+        void SetLabel(char const * label) const;
         void SetPipeline(ComputePipeline const& pipeline) const;
         void WriteTimestamp(QuerySet const& querySet, uint32_t queryIndex) const;
 
@@ -938,11 +981,13 @@ namespace wgpu {
         SwapChain CreateSwapChain(Surface const& surface, SwapChainDescriptor const * descriptor) const;
         Texture CreateTexture(TextureDescriptor const * descriptor) const;
         void Destroy() const;
+        size_t EnumerateFeatures(FeatureName * features) const;
         bool GetLimits(SupportedLimits * limits) const;
         Queue GetQueue() const;
-        bool PopErrorScope(ErrorCallback callback, void * userdata) const;
+        bool HasFeature(FeatureName feature) const;
+        void PopErrorScope(ErrorCallback callback, void * userdata) const;
         void PushErrorScope(ErrorFilter filter) const;
-        void SetDeviceLostCallback(DeviceLostCallback callback, void * userdata) const;
+        void SetLabel(char const * label) const;
         void SetUncapturedErrorCallback(ErrorCallback callback, void * userdata) const;
 
       private:
@@ -971,6 +1016,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<PipelineLayout, WGPUPipelineLayout>;
@@ -984,6 +1030,9 @@ namespace wgpu {
         using ObjectBase::operator=;
 
         void Destroy() const;
+        uint32_t GetCount() const;
+        QueryType GetType() const;
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<QuerySet, WGPUQuerySet>;
@@ -997,7 +1046,8 @@ namespace wgpu {
         using ObjectBase::operator=;
 
         void OnSubmittedWorkDone(uint64_t signalValue, QueueWorkDoneCallback callback, void * userdata) const;
-        void Submit(uint32_t commandCount, CommandBuffer const * commands) const;
+        void SetLabel(char const * label) const;
+        void Submit(size_t commandCount, CommandBuffer const * commands) const;
         void WriteBuffer(Buffer const& buffer, uint64_t bufferOffset, void const * data, size_t size) const;
         void WriteTexture(ImageCopyTexture const * destination, void const * data, size_t dataSize, TextureDataLayout const * dataLayout, Extent3D const * writeSize) const;
 
@@ -1012,6 +1062,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<RenderBundle, WGPURenderBundle>;
@@ -1032,8 +1083,9 @@ namespace wgpu {
         void InsertDebugMarker(char const * markerLabel) const;
         void PopDebugGroup() const;
         void PushDebugGroup(char const * groupLabel) const;
-        void SetBindGroup(uint32_t groupIndex, BindGroup const& group, uint32_t dynamicOffsetCount = 0, uint32_t const * dynamicOffsets = nullptr) const;
+        void SetBindGroup(uint32_t groupIndex, BindGroup const& group, size_t dynamicOffsetCount = 0, uint32_t const * dynamicOffsets = nullptr) const;
         void SetIndexBuffer(Buffer const& buffer, IndexFormat format, uint64_t offset = 0, uint64_t size = WGPU_WHOLE_SIZE) const;
+        void SetLabel(char const * label) const;
         void SetPipeline(RenderPipeline const& pipeline) const;
         void SetVertexBuffer(uint32_t slot, Buffer const& buffer, uint64_t offset = 0, uint64_t size = WGPU_WHOLE_SIZE) const;
 
@@ -1057,13 +1109,14 @@ namespace wgpu {
         void End() const;
         void EndOcclusionQuery() const;
         void EndPipelineStatisticsQuery() const;
-        void ExecuteBundles(uint32_t bundlesCount, RenderBundle const * bundles) const;
+        void ExecuteBundles(size_t bundleCount, RenderBundle const * bundles) const;
         void InsertDebugMarker(char const * markerLabel) const;
         void PopDebugGroup() const;
         void PushDebugGroup(char const * groupLabel) const;
-        void SetBindGroup(uint32_t groupIndex, BindGroup const& group, uint32_t dynamicOffsetCount = 0, uint32_t const * dynamicOffsets = nullptr) const;
+        void SetBindGroup(uint32_t groupIndex, BindGroup const& group, size_t dynamicOffsetCount = 0, uint32_t const * dynamicOffsets = nullptr) const;
         void SetBlendConstant(Color const * color) const;
         void SetIndexBuffer(Buffer const& buffer, IndexFormat format, uint64_t offset = 0, uint64_t size = WGPU_WHOLE_SIZE) const;
+        void SetLabel(char const * label) const;
         void SetPipeline(RenderPipeline const& pipeline) const;
         void SetScissorRect(uint32_t x, uint32_t y, uint32_t width, uint32_t height) const;
         void SetStencilReference(uint32_t reference) const;
@@ -1096,6 +1149,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<Sampler, WGPUSampler>;
@@ -1151,6 +1205,15 @@ namespace wgpu {
 
         TextureView CreateView(TextureViewDescriptor const * descriptor = nullptr) const;
         void Destroy() const;
+        uint32_t GetDepthOrArrayLayers() const;
+        TextureDimension GetDimension() const;
+        TextureFormat GetFormat() const;
+        uint32_t GetHeight() const;
+        uint32_t GetMipLevelCount() const;
+        uint32_t GetSampleCount() const;
+        TextureUsage GetUsage() const;
+        uint32_t GetWidth() const;
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<Texture, WGPUTexture>;
@@ -1163,6 +1226,7 @@ namespace wgpu {
         using ObjectBase::ObjectBase;
         using ObjectBase::operator=;
 
+        void SetLabel(char const * label) const;
 
       private:
         friend ObjectBase<TextureView, WGPUTextureView>;
@@ -1187,11 +1251,14 @@ namespace wgpu {
     struct AdapterProperties {
         ChainedStructOut  * nextInChain = nullptr;
         uint32_t vendorID;
+        char const * vendorName;
+        char const * architecture;
         uint32_t deviceID;
         char const * name;
         char const * driverDescription;
         AdapterType adapterType;
         BackendType backendType;
+        bool compatibilityMode = false;
     };
 
     struct BindGroupEntry {
@@ -1199,7 +1266,7 @@ namespace wgpu {
         uint32_t binding;
         Buffer buffer = nullptr;
         uint64_t offset = 0;
-        uint64_t size;
+        uint64_t size = WGPU_WHOLE_SIZE;
         Sampler sampler = nullptr;
         TextureView textureView = nullptr;
     };
@@ -1250,11 +1317,15 @@ namespace wgpu {
         uint64_t linePos;
         uint64_t offset;
         uint64_t length;
+        uint64_t utf16LinePos;
+        uint64_t utf16Offset;
+        uint64_t utf16Length;
     };
 
-    struct ComputePassDescriptor {
-        ChainedStruct const * nextInChain = nullptr;
-        char const * label = nullptr;
+    struct ComputePassTimestampWrite {
+        QuerySet querySet;
+        uint32_t queryIndex;
+        ComputePassTimestampLocation location;
     };
 
     struct ConstantEntry {
@@ -1279,6 +1350,8 @@ namespace wgpu {
         uint32_t maxTextureDimension3D = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxTextureArrayLayers = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxBindGroups = WGPU_LIMIT_U32_UNDEFINED;
+        uint32_t maxBindGroupsPlusVertexBuffers = WGPU_LIMIT_U32_UNDEFINED;
+        uint32_t maxBindingsPerBindGroup = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxDynamicUniformBuffersPerPipelineLayout = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxDynamicStorageBuffersPerPipelineLayout = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxSampledTexturesPerShaderStage = WGPU_LIMIT_U32_UNDEFINED;
@@ -1291,9 +1364,13 @@ namespace wgpu {
         uint32_t minUniformBufferOffsetAlignment = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t minStorageBufferOffsetAlignment = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxVertexBuffers = WGPU_LIMIT_U32_UNDEFINED;
+        uint64_t maxBufferSize = WGPU_LIMIT_U64_UNDEFINED;
         uint32_t maxVertexAttributes = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxVertexBufferArrayStride = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxInterStageShaderComponents = WGPU_LIMIT_U32_UNDEFINED;
+        uint32_t maxInterStageShaderVariables = WGPU_LIMIT_U32_UNDEFINED;
+        uint32_t maxColorAttachments = WGPU_LIMIT_U32_UNDEFINED;
+        uint32_t maxColorAttachmentBytesPerSample = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxComputeWorkgroupStorageSize = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxComputeInvocationsPerWorkgroup = WGPU_LIMIT_U32_UNDEFINED;
         uint32_t maxComputeWorkgroupSizeX = WGPU_LIMIT_U32_UNDEFINED;
@@ -1318,22 +1395,17 @@ namespace wgpu {
     struct PipelineLayoutDescriptor {
         ChainedStruct const * nextInChain = nullptr;
         char const * label = nullptr;
-        uint32_t bindGroupLayoutCount;
+        size_t bindGroupLayoutCount;
         BindGroupLayout const * bindGroupLayouts;
     };
 
-    struct PrimitiveDepthClampingState : ChainedStruct {
-        PrimitiveDepthClampingState() {
-            sType = SType::PrimitiveDepthClampingState;
-        }
-        alignas(ChainedStruct) bool clampDepth = false;
-    };
-
+    // Can be chained in PrimitiveState
     struct PrimitiveDepthClipControl : ChainedStruct {
         PrimitiveDepthClipControl() {
             sType = SType::PrimitiveDepthClipControl;
         }
-        alignas(ChainedStruct) bool unclippedDepth = false;
+        static constexpr size_t kFirstMemberAlignment = detail::ConstexprMax(alignof(ChainedStruct), alignof(bool ));
+        alignas(kFirstMemberAlignment) bool unclippedDepth = false;
     };
 
     struct PrimitiveState {
@@ -1350,7 +1422,12 @@ namespace wgpu {
         QueryType type;
         uint32_t count;
         PipelineStatisticName const * pipelineStatistics;
-        uint32_t pipelineStatisticsCount = 0;
+        size_t pipelineStatisticsCount = 0;
+    };
+
+    struct QueueDescriptor {
+        ChainedStruct const * nextInChain = nullptr;
+        char const * label = nullptr;
     };
 
     struct RenderBundleDescriptor {
@@ -1361,7 +1438,7 @@ namespace wgpu {
     struct RenderBundleEncoderDescriptor {
         ChainedStruct const * nextInChain = nullptr;
         char const * label = nullptr;
-        uint32_t colorFormatsCount;
+        size_t colorFormatsCount;
         TextureFormat const * colorFormats;
         TextureFormat depthStencilFormat = TextureFormat::Undefined;
         uint32_t sampleCount = 1;
@@ -1373,7 +1450,7 @@ namespace wgpu {
         TextureView view;
         LoadOp depthLoadOp = LoadOp::Undefined;
         StoreOp depthStoreOp = StoreOp::Undefined;
-        float depthClearValue = 0;
+        float depthClearValue = NAN;
         bool depthReadOnly = false;
         LoadOp stencilLoadOp = LoadOp::Undefined;
         StoreOp stencilStoreOp = StoreOp::Undefined;
@@ -1381,11 +1458,28 @@ namespace wgpu {
         bool stencilReadOnly = false;
     };
 
+    // Can be chained in RenderPassDescriptor
+    struct RenderPassDescriptorMaxDrawCount : ChainedStruct {
+        RenderPassDescriptorMaxDrawCount() {
+            sType = SType::RenderPassDescriptorMaxDrawCount;
+        }
+        static constexpr size_t kFirstMemberAlignment = detail::ConstexprMax(alignof(ChainedStruct), alignof(uint64_t ));
+        alignas(kFirstMemberAlignment) uint64_t maxDrawCount = 50000000;
+    };
+
+    struct RenderPassTimestampWrite {
+        QuerySet querySet;
+        uint32_t queryIndex;
+        RenderPassTimestampLocation location;
+    };
+
     struct RequestAdapterOptions {
         ChainedStruct const * nextInChain = nullptr;
         Surface compatibleSurface = nullptr;
         PowerPreference powerPreference = PowerPreference::Undefined;
+        BackendType backendType = BackendType::Undefined;
         bool forceFallbackAdapter = false;
+        bool compatibilityMode = false;
     };
 
     struct SamplerBindingLayout {
@@ -1401,9 +1495,9 @@ namespace wgpu {
         AddressMode addressModeW = AddressMode::ClampToEdge;
         FilterMode magFilter = FilterMode::Nearest;
         FilterMode minFilter = FilterMode::Nearest;
-        FilterMode mipmapFilter = FilterMode::Nearest;
+        MipmapFilterMode mipmapFilter = MipmapFilterMode::Nearest;
         float lodMinClamp = 0.0f;
-        float lodMaxClamp = 1000.0f;
+        float lodMaxClamp = 32.0f;
         CompareFunction compare = CompareFunction::Undefined;
         uint16_t maxAnisotropy = 1;
     };
@@ -1413,19 +1507,23 @@ namespace wgpu {
         char const * label = nullptr;
     };
 
+    // Can be chained in ShaderModuleDescriptor
     struct ShaderModuleSPIRVDescriptor : ChainedStruct {
         ShaderModuleSPIRVDescriptor() {
             sType = SType::ShaderModuleSPIRVDescriptor;
         }
-        alignas(ChainedStruct) uint32_t codeSize;
+        static constexpr size_t kFirstMemberAlignment = detail::ConstexprMax(alignof(ChainedStruct), alignof(uint32_t ));
+        alignas(kFirstMemberAlignment) uint32_t codeSize;
         uint32_t const * code;
     };
 
+    // Can be chained in ShaderModuleDescriptor
     struct ShaderModuleWGSLDescriptor : ChainedStruct {
         ShaderModuleWGSLDescriptor() {
             sType = SType::ShaderModuleWGSLDescriptor;
         }
-        alignas(ChainedStruct) char const * source;
+        static constexpr size_t kFirstMemberAlignment = detail::ConstexprMax(alignof(ChainedStruct), alignof(char const * ));
+        alignas(kFirstMemberAlignment) char const * code;
     };
 
     struct StencilFaceState {
@@ -1447,11 +1545,13 @@ namespace wgpu {
         char const * label = nullptr;
     };
 
+    // Can be chained in SurfaceDescriptor
     struct SurfaceDescriptorFromCanvasHTMLSelector : ChainedStruct {
         SurfaceDescriptorFromCanvasHTMLSelector() {
             sType = SType::SurfaceDescriptorFromCanvasHTMLSelector;
         }
-        alignas(ChainedStruct) char const * selector;
+        static constexpr size_t kFirstMemberAlignment = detail::ConstexprMax(alignof(ChainedStruct), alignof(char const * ));
+        alignas(kFirstMemberAlignment) char const * selector;
     };
 
     struct SwapChainDescriptor {
@@ -1500,7 +1600,7 @@ namespace wgpu {
         ChainedStruct const * nextInChain = nullptr;
         char const * label = nullptr;
         BindGroupLayout layout;
-        uint32_t entryCount;
+        size_t entryCount;
         BindGroupEntry const * entries;
     };
 
@@ -1521,15 +1621,22 @@ namespace wgpu {
 
     struct CompilationInfo {
         ChainedStruct const * nextInChain = nullptr;
-        uint32_t messageCount;
+        size_t messageCount;
         CompilationMessage const * messages;
     };
 
+    struct ComputePassDescriptor {
+        ChainedStruct const * nextInChain = nullptr;
+        char const * label = nullptr;
+        size_t timestampWriteCount = 0;
+        ComputePassTimestampWrite const * timestampWrites;
+    };
+
     struct DepthStencilState {
         ChainedStruct const * nextInChain = nullptr;
         TextureFormat format;
-        bool depthWriteEnabled = false;
-        CompareFunction depthCompare = CompareFunction::Always;
+        bool depthWriteEnabled;
+        CompareFunction depthCompare;
         StencilFaceState stencilFront;
         StencilFaceState stencilBack;
         uint32_t stencilReadMask = 0xFFFFFFFF;
@@ -1557,12 +1664,13 @@ namespace wgpu {
         ChainedStruct const * nextInChain = nullptr;
         ShaderModule module;
         char const * entryPoint;
-        uint32_t constantCount = 0;
+        size_t constantCount = 0;
         ConstantEntry const * constants;
     };
 
     struct RenderPassColorAttachment {
-        TextureView view;
+        ChainedStruct const * nextInChain = nullptr;
+        TextureView view = nullptr;
         TextureView resolveTarget = nullptr;
         LoadOp loadOp;
         StoreOp storeOp;
@@ -1588,19 +1696,21 @@ namespace wgpu {
         TextureFormat format;
         uint32_t mipLevelCount = 1;
         uint32_t sampleCount = 1;
+        size_t viewFormatCount = 0;
+        TextureFormat const * viewFormats;
     };
 
     struct VertexBufferLayout {
         uint64_t arrayStride;
         VertexStepMode stepMode = VertexStepMode::Vertex;
-        uint32_t attributeCount;
+        size_t attributeCount;
         VertexAttribute const * attributes;
     };
 
     struct BindGroupLayoutDescriptor {
         ChainedStruct const * nextInChain = nullptr;
         char const * label = nullptr;
-        uint32_t entryCount;
+        size_t entryCount;
         BindGroupLayoutEntry const * entries;
     };
 
@@ -1621,27 +1731,32 @@ namespace wgpu {
     struct DeviceDescriptor {
         ChainedStruct const * nextInChain = nullptr;
         char const * label = nullptr;
-        uint32_t requiredFeaturesCount = 0;
+        size_t requiredFeaturesCount = 0;
         FeatureName const * requiredFeatures = nullptr;
         RequiredLimits const * requiredLimits = nullptr;
+        QueueDescriptor defaultQueue;
+        DeviceLostCallback deviceLostCallback = nullptr;
+        void * deviceLostUserdata = nullptr;
     };
 
     struct RenderPassDescriptor {
         ChainedStruct const * nextInChain = nullptr;
         char const * label = nullptr;
-        uint32_t colorAttachmentCount;
+        size_t colorAttachmentCount;
         RenderPassColorAttachment const * colorAttachments;
         RenderPassDepthStencilAttachment const * depthStencilAttachment = nullptr;
         QuerySet occlusionQuerySet = nullptr;
+        size_t timestampWriteCount = 0;
+        RenderPassTimestampWrite const * timestampWrites;
     };
 
     struct VertexState {
         ChainedStruct const * nextInChain = nullptr;
         ShaderModule module;
         char const * entryPoint;
-        uint32_t constantCount = 0;
+        size_t constantCount = 0;
         ConstantEntry const * constants;
-        uint32_t bufferCount = 0;
+        size_t bufferCount = 0;
         VertexBufferLayout const * buffers;
     };
 
@@ -1649,9 +1764,9 @@ namespace wgpu {
         ChainedStruct const * nextInChain = nullptr;
         ShaderModule module;
         char const * entryPoint;
-        uint32_t constantCount = 0;
+        size_t constantCount = 0;
         ConstantEntry const * constants;
-        uint32_t targetCount;
+        size_t targetCount;
         ColorTargetState const * targets;
     };
 
