@@ -36,6 +36,7 @@
 #include <emscripten/threading.h>
 
 #include "threading_internal.h"
+#include "emscripten_internal.h"
 
 int emscripten_pthread_attr_gettransferredcanvases(const pthread_attr_t* a, const char** str) {
   *str = a->_a_transferredcanvases;
@@ -176,9 +177,6 @@ void emscripten_async_waitable_close(em_queued_call* call) {
   em_queued_call_free(call);
 }
 
-extern EMSCRIPTEN_RESULT _emscripten_set_offscreencanvas_size(const char *target, int width, int height);
-extern double emscripten_receive_on_main_thread_js(int functionIndex, int numCallArgs, double* args);
-
 static void _do_call(void* arg) {
   em_queued_call* q = (em_queued_call*)arg;
   // C function pointer
@@ -190,7 +188,7 @@ static void _do_call(void* arg) {
       break;
     case EM_PROXIED_JS_FUNCTION:
       q->returnValue.d =
-        emscripten_receive_on_main_thread_js((int)(size_t)q->functionPtr, q->args[0].i, &q->args[1].d);
+        emscripten_receive_on_main_thread_js((intptr_t)q->functionPtr, q->callingThread, q->args[0].i, &q->args[1].d);
       break;
     case EM_FUNC_SIG_V:
       ((em_func_v)q->functionPtr)();
@@ -352,14 +350,14 @@ EMSCRIPTEN_RESULT emscripten_wait_for_call_i(
 
 static struct pthread __main_pthread;
 
-pthread_t emscripten_main_browser_thread_id() {
+pthread_t emscripten_main_runtime_thread_id() {
   return &__main_pthread;
 }
 
 static pthread_t normalize_thread(pthread_t target_thread) {
   assert(target_thread);
-  if (target_thread == EM_CALLBACK_THREAD_CONTEXT_MAIN_BROWSER_THREAD) {
-    return emscripten_main_browser_thread_id();
+  if (target_thread == EM_CALLBACK_THREAD_CONTEXT_MAIN_RUNTIME_THREAD) {
+    return emscripten_main_runtime_thread_id();
   }
   if (target_thread == EM_CALLBACK_THREAD_CONTEXT_CALLING_THREAD) {
     return pthread_self();
@@ -392,119 +390,22 @@ static int do_dispatch_to_thread(pthread_t target_thread,
 }
 
 void emscripten_async_run_in_main_thread(em_queued_call* call) {
-  do_dispatch_to_thread(emscripten_main_browser_thread_id(), call);
+  do_dispatch_to_thread(emscripten_main_runtime_thread_id(), call);
 }
 
-void emscripten_sync_run_in_main_thread(em_queued_call* call) {
+static void sync_run_in_main_thread(em_queued_call* call) {
   emscripten_async_run_in_main_thread(call);
 
   // Enter to wait for the operation to complete.
   emscripten_wait_for_call_v(call, INFINITY);
 }
 
-void* emscripten_sync_run_in_main_thread_0(int function) {
-  em_queued_call q = {function};
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
-void* emscripten_sync_run_in_main_thread_1(int function, void* arg1) {
-  em_queued_call q = {function};
-  q.args[0].vp = arg1;
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
-void* emscripten_sync_run_in_main_thread_2(
-  int function, void* arg1, void* arg2) {
-  em_queued_call q = {function};
-  q.args[0].vp = arg1;
-  q.args[1].vp = arg2;
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
-void* emscripten_sync_run_in_main_thread_3(
-  int function, void* arg1, void* arg2, void* arg3) {
-  em_queued_call q = {function};
-  q.args[0].vp = arg1;
-  q.args[1].vp = arg2;
-  q.args[2].vp = arg3;
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
-void* emscripten_sync_run_in_main_thread_4(
-  int function, void* arg1, void* arg2, void* arg3, void* arg4) {
-  em_queued_call q = {function};
-  q.args[0].vp = arg1;
-  q.args[1].vp = arg2;
-  q.args[2].vp = arg3;
-  q.args[3].vp = arg4;
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
-void* emscripten_sync_run_in_main_thread_5(
-  int function, void* arg1, void* arg2, void* arg3, void* arg4, void* arg5) {
-  em_queued_call q = {function};
-  q.args[0].vp = arg1;
-  q.args[1].vp = arg2;
-  q.args[2].vp = arg3;
-  q.args[3].vp = arg4;
-  q.args[4].vp = arg5;
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
-void* emscripten_sync_run_in_main_thread_6(
-  int function, void* arg1, void* arg2, void* arg3, void* arg4, void* arg5, void* arg6) {
-  em_queued_call q = {function};
-  q.args[0].vp = arg1;
-  q.args[1].vp = arg2;
-  q.args[2].vp = arg3;
-  q.args[3].vp = arg4;
-  q.args[4].vp = arg5;
-  q.args[5].vp = arg6;
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
-void* emscripten_sync_run_in_main_thread_7(int function, void* arg1,
-  void* arg2, void* arg3, void* arg4, void* arg5, void* arg6, void* arg7) {
-  em_queued_call q = {function};
-  q.args[0].vp = arg1;
-  q.args[1].vp = arg2;
-  q.args[2].vp = arg3;
-  q.args[3].vp = arg4;
-  q.args[4].vp = arg5;
-  q.args[5].vp = arg6;
-  q.args[6].vp = arg7;
-  q.returnValue.vp = 0;
-  emscripten_sync_run_in_main_thread(&q);
-  return q.returnValue.vp;
-}
-
 void emscripten_current_thread_process_queued_calls() {
   emscripten_proxy_execute_queue(emscripten_proxy_get_system_queue());
 }
 
-// At times when we disallow the main thread to process queued calls, this will
-// be set to 0.
-int _emscripten_allow_main_runtime_queued_calls = 1;
-
 void emscripten_main_thread_process_queued_calls() {
   assert(emscripten_is_main_runtime_thread());
-  if (!_emscripten_allow_main_runtime_queued_calls)
-    return;
-
   emscripten_current_thread_process_queued_calls();
 }
 
@@ -515,11 +416,11 @@ int emscripten_sync_run_in_main_runtime_thread_(EM_FUNC_SIGNATURE sig, void* fun
   va_start(args, func_ptr);
   init_em_queued_call_args(&q, sig, args);
   va_end(args);
-  emscripten_sync_run_in_main_thread(&q);
+  sync_run_in_main_thread(&q);
   return q.returnValue.i;
 }
 
-double emscripten_run_in_main_runtime_thread_js(int index, int num_args, int64_t* buffer, int sync) {
+double _emscripten_run_in_main_runtime_thread_js(int index, int num_args, int64_t* buffer, int sync) {
   em_queued_call q;
   em_queued_call *c;
   if (sync) {
@@ -529,10 +430,11 @@ double emscripten_run_in_main_runtime_thread_js(int index, int num_args, int64_t
   } else {
     c = em_queued_call_malloc();
   }
-  c->calleeDelete = 1-sync;
+  c->calleeDelete = !sync;
   c->functionEnum = EM_PROXIED_JS_FUNCTION;
   // Index not needed to ever be more than 32-bit.
-  c->functionPtr = (void*)(size_t)index;
+  c->functionPtr = (void*)(intptr_t)index;
+  c->callingThread = pthread_self();
   assert(num_args+1 <= EM_QUEUED_JS_CALL_MAX_ARGS);
   // The types are only known at runtime in these calls, so we store values that
   // must be able to contain any valid JS value, including a 64-bit BigInt if
@@ -544,7 +446,7 @@ double emscripten_run_in_main_runtime_thread_js(int index, int num_args, int64_t
   }
 
   if (sync) {
-    emscripten_sync_run_in_main_thread(&q);
+    sync_run_in_main_thread(&q);
     // TODO: support BigInt return values somehow.
     return q.returnValue.d;
   } else {
@@ -664,6 +566,10 @@ int emscripten_dispatch_to_thread_async_(pthread_t target_thread,
   return ret;
 }
 
+int _emscripten_thread_is_valid(pthread_t thread) {
+  return thread->self == thread;
+}
+
 static void *dummy_tsd[1] = { 0 };
 weak_alias(dummy_tsd, __pthread_tsd_main);
 
@@ -684,7 +590,11 @@ void __emscripten_init_main_thread(void) {
   // tid is always non-zero.
   __main_pthread.tid = getpid();
   __main_pthread.locale = &libc.global_locale;
-  // TODO(sbc): Implement circular list of threads
-  //__main_pthread.next = __main_pthread.prev = &__main_pthread;
+  // pthread struct prev and next should initially point to itself (see __init_tp),
+  // this is used by pthread_key_delete for deleting thread-specific data.
+  __main_pthread.next = __main_pthread.prev = &__main_pthread;
   __main_pthread.tsd = (void **)__pthread_tsd_main;
+
+  _emscripten_thread_mailbox_init(&__main_pthread);
+  _emscripten_thread_mailbox_await(&__main_pthread);
 }
