@@ -5,8 +5,10 @@
  * found in the LICENSE file.
  */
 
+#define _GNU_SOURCE
 #include <assert.h>
 #include <errno.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/mman.h>
@@ -17,10 +19,12 @@
 
 #include <emscripten.h>
 #include <emscripten/heap.h>
+#include <emscripten/console.h>
 #include <wasi/api.h>
 #include <wasi/wasi-helpers.h>
 
 #include "lock.h"
+#include "emscripten_internal.h"
 
 /*
  * WASI support code. These are compiled with the program, and call out
@@ -66,18 +70,18 @@ int clock_getres(clockid_t clk_id, struct timespec *tp) {
 
 // Mark these as weak so that wasmfs does not collide with it. That is, if
 // wasmfs is in use, we want to use that and not this.
-__attribute__((__weak__)) int _mmap_js(size_t length,
-                                       int prot,
-                                       int flags,
-                                       int fd,
-                                       size_t offset,
-                                       int* allocated,
-                                       void** addr) {
+weak int _mmap_js(size_t length,
+                  int prot,
+                  int flags,
+                  int fd,
+                  off_t offset,
+                  int* allocated,
+                  void** addr) {
   return -ENOSYS;
 }
 
-__attribute__((__weak__)) int _munmap_js(
-  intptr_t addr, size_t length, int prot, int flags, int fd, size_t offset) {
+weak int _munmap_js(
+  intptr_t addr, size_t length, int prot, int flags, int fd, off_t offset) {
   return -ENOSYS;
 }
 
@@ -85,8 +89,7 @@ __attribute__((__weak__)) int _munmap_js(
 // corner case error checking; everything else is not permitted.
 // TODO: full file support for WASI, or an option for it
 // open()
-__attribute__((__weak__))
-int __syscall_openat(int dirfd, intptr_t path, int flags, ...) {
+weak int __syscall_openat(int dirfd, intptr_t path, int flags, ...) {
   if (!strcmp((const char*)path, "/dev/stdin")) {
     return STDIN_FILENO;
   }
@@ -99,31 +102,48 @@ int __syscall_openat(int dirfd, intptr_t path, int flags, ...) {
   return -EPERM;
 }
 
-__attribute__((__weak__)) int __syscall_ioctl(int fd, int op, ...) {
+weak int __syscall_ioctl(int fd, int op, ...) {
   return -ENOSYS;
 }
 
-__attribute__((__weak__)) int __syscall_fcntl64(int fd, int cmd, ...) {
+weak int __syscall_fcntl64(int fd, int cmd, ...) {
   return -ENOSYS;
 }
 
-__attribute__((__weak__)) int __syscall_fstat64(int fd, intptr_t buf) {
+weak int __syscall_fstat64(int fd, intptr_t buf) {
+  return -ENOSYS;
+}
+
+weak int __syscall_stat64(intptr_t path, intptr_t buf) {
+  return -ENOSYS;
+}
+
+weak int __syscall_dup(int fd) {
+  return -ENOSYS;
+}
+
+weak int __syscall_mkdirat(int dirfd, intptr_t path, int mode) {
+  return -ENOSYS;
+}
+
+weak int __syscall_newfstatat(int dirfd, intptr_t path, intptr_t buf, int flags) {
+  return -ENOSYS;
+}
+
+weak int __syscall_lstat64(intptr_t path, intptr_t buf) {
   return -ENOSYS;
 }
 
 // There is no good source of entropy without an import. Make this weak so that
 // it can be replaced with a pRNG or a proper import.
-__attribute__((__weak__))
-int getentropy(void* buffer, size_t length) {
+weak int getentropy(void* buffer, size_t length) {
   abort();
 }
 
 // Emscripten additions
 
-extern void emscripten_notify_memory_growth(size_t memory_index);
-
 // Should never be called in standalone mode
-void emscripten_memcpy_big(void *restrict dest, const void *restrict src, size_t n) {
+weak void emscripten_memcpy_big(void *restrict dest, const void *restrict src, size_t n) {
   __builtin_unreachable();
 }
 
@@ -140,7 +160,10 @@ int emscripten_resize_heap(size_t size) {
   assert(old_size < size);
   ssize_t diff = (size - old_size + WASM_PAGE_SIZE - 1) / WASM_PAGE_SIZE;
   size_t result = __builtin_wasm_memory_grow(0, diff);
-  if (result != (size_t)-1) {
+  // Its seems v8 has a bug in memory.grow that causes it to return 
+  // (uint32_t)-1 even with memory64:
+  // https://bugs.chromium.org/p/v8/issues/detail?id=13948
+  if (result != (uint32_t)-1 && result != (size_t)-1) {
     // Success, update JS (see https://github.com/WebAssembly/WASI/issues/82)
     emscripten_notify_memory_growth(0);
     return 1;
@@ -149,30 +172,24 @@ int emscripten_resize_heap(size_t size) {
   return 0;
 }
 
-double emscripten_get_now(void) {
-  return (1000 * clock()) / (double)CLOCKS_PER_SEC;
+weak double emscripten_get_now(void) {
+  struct timespec ts;
+  if (clock_gettime(CLOCK_MONOTONIC, &ts)) {
+    return 0;
+  }
+  // emscripten_get_now returns time in milliseconds (as a double)
+  return (double)ts.tv_sec * 1000 + (double)ts.tv_nsec / 1000000;
 }
 
 // C++ ABI
 
-// Emscripten disables exception catching by default, but not throwing. That
-// allows users to see a clear error if a throw happens, and 99% of the
-// overhead is in the catching, so this is a reasonable tradeoff.
-// For now, in a standalone build just terminate. TODO nice error message
-//
-// Define these symbols as weak so that when we build with exceptions
-// enabled (using wasm-eh) we get the real versions of these functions
-// as defined in libc++abi.
-
-__attribute__((__weak__))
+#if EMSCRIPTEN_NOCATCH
+// When exception catching is disabled, we stub out calls to `__cxa_throw`.
+// Otherwise, `__cxa_throw` will be imported from the host.
 void __cxa_throw(void* ptr, void* type, void* destructor) {
   abort();
 }
-
-__attribute__((__weak__))
-void* __cxa_allocate_exception(size_t thrown_size) {
-  abort();
-}
+#endif
 
 // WasmFS integration. We stub out file preloading and such, that are not
 // expected to work anyhow.
@@ -206,16 +223,110 @@ imported__wasi_fd_write(__wasi_fd_t fd,
                         __wasi_size_t* nwritten);
 
 // Write a buffer + a newline.
-static void wasi_writeln(__wasi_fd_t fd, char* buffer) {
+static void wasi_writeln_n(__wasi_fd_t fd, const char* buffer, size_t len) {
   struct __wasi_ciovec_t iovs[2];
   iovs[0].buf = (uint8_t*)buffer;
-  iovs[0].buf_len = strlen(buffer);
+  iovs[0].buf_len = len;
   iovs[1].buf = (uint8_t*)"\n";
   iovs[1].buf_len = 1;
   __wasi_size_t nwritten;
   imported__wasi_fd_write(fd, iovs, 2, &nwritten);
 }
 
-void _emscripten_out(char* text) { wasi_writeln(1, text); }
+static void wasi_writeln(__wasi_fd_t fd, const char* buffer) {
+  return wasi_writeln_n(fd, buffer, strlen(buffer));
+}
 
-void _emscripten_err(char* text) { wasi_writeln(2, text); }
+weak void emscripten_out(const char* text) { wasi_writeln(1, text); }
+
+weak void emscripten_err(const char* text) { wasi_writeln(2, text); }
+
+weak void emscripten_dbg(const char* text) { wasi_writeln(2, text); }
+
+weak void emscripten_outn(const char* text, size_t len) {
+  wasi_writeln_n(1, text, len);
+}
+
+weak void emscripten_errn(const char* text, size_t len) {
+  wasi_writeln_n(2, text, len);
+}
+
+weak void emscripten_dbgn(const char* text, size_t len) {
+  wasi_writeln_n(2, text, len);
+}
+
+__attribute__((import_module("wasi_snapshot_preview1"),
+               import_name("fd_read"))) __wasi_errno_t
+imported__wasi_fd_read(__wasi_fd_t fd,
+                        const __wasi_ciovec_t* iovs,
+                        size_t iovs_len,
+                        __wasi_size_t* nread);
+
+int _wasmfs_stdin_get_char(void) {
+  char c;
+  struct __wasi_ciovec_t iov;
+  iov.buf = (uint8_t*)&c;
+  iov.buf_len = 1;
+  __wasi_size_t nread;
+  imported__wasi_fd_read(0, &iov, 1, &nread);
+  if (nread == 0) {
+    return -1;
+  }
+  return c;
+}
+
+// In the non-standalone build we define this helper function in JS to avoid
+// signture mismatch issues.
+// See: https://github.com/emscripten-core/posixtestsuite/issues/6
+void __call_sighandler(sighandler_t handler, int sig) {
+  handler(sig);
+}
+
+int _setitimer_js(int which, double timeout) {
+  // There is no API to let us set timers in standalone mode atm. Return an
+  // error.
+  errno = ENOTSUP;
+  return -1;
+}
+
+weak uintptr_t emscripten_stack_snapshot(void) {
+  return 0;
+}
+
+weak uint32_t emscripten_stack_unwind_buffer(uintptr_t pc,
+                                             uintptr_t* buffer,
+                                             uint32_t depth) {
+  return 0;
+}
+
+weak const char* emscripten_pc_get_function(uintptr_t pc) {
+  return NULL;
+}
+
+weak const char* emscripten_pc_get_file(uintptr_t pc) {
+  return NULL;
+}
+
+weak int emscripten_pc_get_line(uintptr_t pc) {
+  return 0;
+}
+
+weak int emscripten_pc_get_column(uintptr_t pc) {
+  return 0;
+}
+
+weak void* emscripten_return_address(int level) {
+  return NULL;
+}
+
+weak int _emscripten_sanitizer_use_colors(void) {
+  return 1;
+}
+
+weak char* _emscripten_sanitizer_get_option(const char* name) {
+  return strdup("");
+}
+
+weak char* emscripten_get_module_name(char* buf, size_t length) {
+  return strncpy(buf, "<unknown>", length);
+}
