@@ -465,22 +465,23 @@ function getWasmImportsValue(node) {
   return node.declarations[0].init;
 }
 
-function isAsmUse(node) {
+function isExportUse(node) {
+  // MINIMAL_RUNTIME calls exports global variable `asm`, whereas regualar
+  // runtime calls it `wasmExports`.
+  //
+  // Here we match either:
+  //   wasmExports['X']
+  // or:
+  //   asm['X']
   return (
     node.type === 'MemberExpression' &&
-    ((node.object.type === 'Identifier' && // asm['X']
-      node.object.name === 'asm' &&
-      node.property.type === 'Literal') ||
-      (node.object.type === 'MemberExpression' && // Module['asm']['X']
-        node.object.object.type === 'Identifier' &&
-        node.object.object.name === 'Module' &&
-        node.object.property.type === 'Literal' &&
-        node.object.property.value === 'asm' &&
-        isLiteralString(node.property)))
+    node.object.type === 'Identifier' &&
+    isLiteralString(node.property) &&
+    (node.object.name === 'wasmExports' || node.object.name === 'asm')
   );
 }
 
-function getAsmOrModuleUseName(node) {
+function getExportOrModuleUseName(node) {
   return node.property.value;
 }
 
@@ -493,19 +494,6 @@ function isModuleUse(node) {
   );
 }
 
-function isModuleAsmUse(node) {
-  // Module['asm'][..string..]
-  return (
-    node.type === 'MemberExpression' &&
-    node.object.type === 'MemberExpression' &&
-    node.object.object.type === 'Identifier' &&
-    node.object.object.name === 'Module' &&
-    node.object.property.type === 'Literal' &&
-    node.object.property.value === 'asm' &&
-    isLiteralString(node.property)
-  );
-}
-
 // Apply import/export name changes (after minifying them)
 function applyImportAndExportNameChanges(ast) {
   const mapping = extraInfo.mapping;
@@ -513,33 +501,27 @@ function applyImportAndExportNameChanges(ast) {
     if (isWasmImportsAssign(node)) {
       const assignedObject = getWasmImportsValue(node);
       assignedObject.properties.forEach((item) => {
-        if (mapping[item.key.value]) {
-          setLiteralValue(item.key, mapping[item.key.value]);
+        if (mapping[item.key.name]) {
+          item.key.name = mapping[item.key.name];
         }
       });
     } else if (node.type === 'AssignmentExpression') {
       const target = node.left;
       const value = node.right;
-      if (isAsmUse(value)) {
+      if (isExportUse(value)) {
         const name = value.property.value;
         if (mapping[name]) {
           setLiteralValue(value.property, mapping[name]);
         }
       }
-    } else if (node.type === 'CallExpression' && isAsmUse(node.callee)) {
+    } else if (node.type === 'CallExpression' && isExportUse(node.callee)) {
       // asm["___wasm_call_ctors"](); -> asm["M"]();
       const callee = node.callee;
       const name = callee.property.value;
       if (mapping[name]) {
         setLiteralValue(callee.property, mapping[name]);
       }
-    } else if (isModuleAsmUse(node)) {
-      const prop = node.property;
-      const name = prop.value;
-      if (mapping[name]) {
-        setLiteralValue(prop, mapping[name]);
-      }
-    } else if (isAsmUse(node)) {
+    } else if (isExportUse(node)) {
       const prop = node.property;
       const name = prop.value;
       if (mapping[name]) {
@@ -585,32 +567,18 @@ function isDynamicDynCall(node) {
 // Matches the wasm export wrappers generated by emcc (see make_export_wrappers
 // in emscripten.py). For example:
 //
-//   var _foo = function() {
-//     return (_foo = Module['asm']['foo']).apply(null, argument)
-//   }
+//   var _foo = (a0, a1) => (_foo = wasmExports['foo'])(a0, a1):
 //
 function isExportWrapperFunction(f) {
-  if (f.body.type != 'BlockStatement') return null;
-  if (f.body.body.length != 1) return null;
-  const expr = f.body.body[0];
-  if (expr.type == 'ReturnStatement') {
-    const rtn = expr.argument;
-    // We are looking for a call of special target, like (x = y)(), and not a
-    // non-call or a normal direct call such as z().
-    if (rtn.type == 'CallExpression' && rtn.callee.object) {
-      let target = rtn.callee.object;
-      if (target.type == 'ParenthesizedExpression') {
-        target = target.expression;
-      }
-      if (target.type == 'AssignmentExpression') {
-        const rhs = target.right;
-        if (isAsmUse(rhs)) {
-          return getAsmOrModuleUseName(rhs);
-        }
-      }
-    }
+  if (f.body.type != 'CallExpression') return null;
+  let callee = f.body.callee;
+  if (callee.type == 'ParenthesizedExpression') {
+    callee = callee.expression;
   }
-  return null;
+  if (callee.type != 'AssignmentExpression' || callee.right.type != 'MemberExpression') return null;
+  var rhs = callee.right;
+  if (rhs.type != 'MemberExpression' || !isExportUse(rhs)) return null;
+  return getExportOrModuleUseName(rhs);
 }
 
 //
@@ -665,17 +633,15 @@ function emitDCEGraph(ast) {
   // The exports are trickier, as they have a different form whether or not
   // async compilation is enabled. It can be either:
   //
-  //  var _malloc = Module["_malloc"] = asm["_malloc"];
+  //  var _malloc = Module['_malloc'] = asm['_malloc'];
   //
   // or
   //
-  //  var _malloc = asm["_malloc"];
+  //  var _malloc = asm['_malloc'];
   //
   // or
   //
-  //  var _malloc = Module["_malloc"] = (function() {
-  //   return Module["asm"]["_malloc"].apply(null, arguments);
-  //  });
+  //  var _malloc = Module['_malloc'] = (x) => wasmExports['_malloc'](x);
   //
   // or, in the minimal runtime, it looks like
   //
@@ -731,15 +697,17 @@ function emitDCEGraph(ast) {
         const item = node.declarations[0];
         const name = item.id.name;
         const value = item.init;
-        if (value && isAsmUse(value)) {
-          const asmName = getAsmOrModuleUseName(value);
+        if (value && isExportUse(value)) {
+          const asmName = getExportOrModuleUseName(value);
           // this is
           //  var _x = asm['x'];
+          // or
+          //  var _x = wasmExports['x'];
           saveAsmExport(name, asmName);
           emptyOut(node);
-        } else if (value && value.type === 'FunctionExpression') {
+        } else if (value && value.type === 'ArrowFunctionExpression') {
           // this is
-          //  var x = function() { return (x = Module['asm']['x']).apply .. }
+          //  var x = () => (x = wasmExports['x'])(..)
           let asmName = isExportWrapperFunction(value);
           if (asmName) {
             saveAsmExport(name, asmName);
@@ -747,16 +715,16 @@ function emitDCEGraph(ast) {
           }
         } else if (value && value.type === 'AssignmentExpression') {
           const assigned = value.left;
-          if (isModuleUse(assigned) && getAsmOrModuleUseName(assigned) === name) {
+          if (isModuleUse(assigned) && getExportOrModuleUseName(assigned) === name) {
             // this is
             //  var x = Module['x'] = ?
             // which looks like a wasm export being received. confirm with the asm use
             let found = 0;
             let asmName;
             fullWalk(value.right, (node) => {
-              if (isAsmUse(node)) {
+              if (isExportUse(node)) {
                 found++;
-                asmName = getAsmOrModuleUseName(node);
+                asmName = getExportOrModuleUseName(node);
               }
             });
             // in the wasm backend, the asm name may have one fewer "_" prefixed
@@ -905,7 +873,7 @@ function emitDCEGraph(ast) {
         reached = nameToGraphName[name];
       }
     } else if (isModuleUse(node)) {
-      const name = getAsmOrModuleUseName(node);
+      const name = getExportOrModuleUseName(node);
       if (modulePropertyToGraphName.hasOwnProperty(name)) {
         reached = modulePropertyToGraphName[name];
       }
@@ -914,9 +882,9 @@ function emitDCEGraph(ast) {
     } else if (isDynamicDynCall(node)) {
       // this can reach *all* dynCall_* targets, we can't narrow it down
       reached = dynCallNames;
-    } else if (isAsmUse(node)) {
+    } else if (isExportUse(node)) {
       // any remaining asm uses are always rooted in any case
-      const name = getAsmOrModuleUseName(node);
+      const name = getExportOrModuleUseName(node);
       if (exportNameToGraphName.hasOwnProperty(name)) {
         infos[exportNameToGraphName[name]].root = true;
       }
@@ -982,7 +950,7 @@ function applyDCEGraphRemovals(ast) {
     if (isWasmImportsAssign(node)) {
       const assignedObject = getWasmImportsValue(node);
       assignedObject.properties = assignedObject.properties.filter((item) => {
-        const name = item.key.value;
+        const name = item.key.name;
         const value = item.value;
         const full = 'emcc$import$' + name;
         return !(unused.has(full) && !hasSideEffects(value));
@@ -991,11 +959,11 @@ function applyDCEGraphRemovals(ast) {
       // when we assign to a thing we don't need, we can just remove the assign
       //   var x = Module['x'] = asm['x'];
       const target = node.left;
-      if (isAsmUse(target) || isModuleUse(target)) {
-        const name = getAsmOrModuleUseName(target);
+      if (isExportUse(target) || isModuleUse(target)) {
+        const name = getExportOrModuleUseName(target);
         const full = 'emcc$export$' + name;
         const value = node.right;
-        if (unused.has(full) && (isAsmUse(value) || !hasSideEffects(value))) {
+        if (unused.has(full) && (isExportUse(value) || !hasSideEffects(value))) {
           // This will be in a var init, and we just remove that value.
           convertToNothingInVarInit(node);
         }
@@ -1007,13 +975,13 @@ function applyDCEGraphRemovals(ast) {
       //   var x = function() { return (x = asm['x']).apply(...) };
       const init = node.declarations[0].init;
       if (init) {
-        if (isAsmUse(init)) {
-          const name = getAsmOrModuleUseName(init);
+        if (isExportUse(init)) {
+          const name = getExportOrModuleUseName(init);
           const full = 'emcc$export$' + name;
           if (unused.has(full)) {
             convertToNothingInVarInit(init);
           }
-        } else if (init.type == 'FunctionExpression') {
+        } else if (init.type == 'ArrowFunctionExpression') {
           const name = isExportWrapperFunction(init);
           const full = 'emcc$export$' + name;
           if (unused.has(full)) {
@@ -1026,9 +994,9 @@ function applyDCEGraphRemovals(ast) {
       // In the minimal runtime code pattern we have just
       //   x = asm['x']
       // and never in a var.
-      if (expr.operator === '=' && expr.left.type === 'Identifier' && isAsmUse(expr.right)) {
+      if (expr.operator === '=' && expr.left.type === 'Identifier' && isExportUse(expr.right)) {
         const name = expr.left.name;
-        if (name === getAsmOrModuleUseName(expr.right)) {
+        if (name === getExportOrModuleUseName(expr.right)) {
           const full = 'emcc$export$' + name;
           if (unused.has(full)) {
             emptyOut(node);
@@ -1945,24 +1913,16 @@ function reattachComments(ast, comments) {
       symbols[j].start.comments_before = [];
     }
     symbols[j].start.comments_before.push(
-      new terser.AST_Token({
-        end: undefined,
-        quote: undefined,
-        raw: undefined,
-        file: '0',
-        comments_after: undefined,
-        comments_before: undefined,
-        nlb: false,
-        endpos: undefined,
-        endcol: undefined,
-        endline: undefined,
-        pos: undefined,
-        col: undefined,
-        line: undefined,
-        value: comments[i].value,
-        type: comments[i].type == 'Line' ? 'comment' : 'comment2',
-        flags: 0,
-      })
+      new terser.AST_Token(
+        comments[i].type == 'Line' ? 'comment' : 'comment2',
+        comments[i].value,
+        undefined,
+        undefined,
+        false,
+        undefined,
+        undefined,
+        '0'
+      )
     );
   }
 }
@@ -2075,6 +2035,7 @@ if (!noPrint) {
     beautify: !minifyWhitespace,
     indent_level: minifyWhitespace ? 0 : 1,
     keep_quoted_props: true, // for closure
+    wrap_func_args: false, // don't add extra braces
     comments: true, // for closure as well
   });
 
