@@ -5,7 +5,9 @@
 
 import contextlib
 import os
+import shutil
 import sys
+from pathlib import Path
 
 from . import diagnostics
 
@@ -20,13 +22,20 @@ def exit_with_error(msg, *args):
 
 
 def path_from_root(*pathelems):
-  return os.path.join(__rootpath__, *pathelems)
+  return str(Path(__rootpath__, *pathelems))
 
 
 def safe_ensure_dirs(dirname):
   os.makedirs(dirname, exist_ok=True)
 
 
+# TODO(sbc): Replace with str.removeprefix once we update to python3.9
+def removeprefix(string, prefix):
+  if string.startswith(prefix):
+    return string[len(prefix):]
+  return string
+
+
 @contextlib.contextmanager
 def chdir(dir):
   """A context manager that performs actions in the given directory."""
@@ -38,41 +47,9 @@ def chdir(dir):
     os.chdir(orig_cwd)
 
 
-# Finds the given executable 'program' in PATH. Operates like the Unix tool 'which'.
-def which(program):
-  def is_exe(fpath):
-    return os.path.isfile(fpath) and os.access(fpath, os.X_OK)
-
-  if os.path.isabs(program):
-    if os.path.isfile(program):
-      return program
-
-    if WINDOWS:
-      for suffix in ['.exe', '.cmd', '.bat']:
-        if is_exe(program + suffix):
-          return program + suffix
-
-  fpath, fname = os.path.split(program)
-  if fpath:
-    if is_exe(program):
-      return program
-  else:
-    for path in os.environ["PATH"].split(os.pathsep):
-      path = path.strip('"')
-      exe_file = os.path.join(path, program)
-      if is_exe(exe_file):
-        return exe_file
-      if WINDOWS:
-        for suffix in ('.exe', '.cmd', '.bat'):
-          if is_exe(exe_file + suffix):
-            return exe_file + suffix
-
-  return None
-
-
 def read_file(file_path):
   """Read from a file opened in text mode"""
-  with open(file_path) as fh:
+  with open(file_path, encoding='utf-8') as fh:
     return fh.read()
 
 
@@ -84,5 +61,40 @@ def read_binary(file_path):
 
 def write_file(file_path, text):
   """Write to a file opened in text mode"""
-  with open(file_path, 'w') as fh:
+  with open(file_path, 'w', encoding='utf-8') as fh:
     fh.write(text)
+
+
+def write_binary(file_path, contents):
+  """Write to a file opened in binary mode"""
+  with open(file_path, 'wb') as fh:
+    fh.write(contents)
+
+
+def delete_file(filename):
+  """Delete a file (if it exists)."""
+  if not os.path.exists(filename):
+    return
+  os.remove(filename)
+
+
+def delete_dir(dirname):
+  """Delete a directory (if it exists)."""
+  if not os.path.exists(dirname):
+    return
+  shutil.rmtree(dirname)
+
+
+def delete_contents(dirname, exclude=None):
+  """Delete the contents of a directory without removing
+  the directory itself."""
+  if not os.path.exists(dirname):
+    return
+  for entry in os.listdir(dirname):
+    if exclude and entry in exclude:
+      continue
+    entry = os.path.join(dirname, entry)
+    if os.path.isdir(entry):
+      delete_dir(entry)
+    else:
+      delete_file(entry)
